import { createLowlight, all } from 'lowlight';
import * as React from 'react';
import { useRef, useEffect, createContext, useContext, useLayoutEffect, useState, useCallback, memo, Fragment, useMemo, forwardRef, useImperativeHandle } from 'react';
import { createStore, ref } from 'reactivity-store';
import { useSyncExternalStore } from 'use-sync-external-store/shim';
import { flushSync } from 'react-dom';

/******************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */
/* global Reflect, Promise, SuppressedError, Symbol */


function __rest(s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
}

function __classPrivateFieldGet$1(receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
}

function __classPrivateFieldSet$1(receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
}

typeof SuppressedError === "function" ? SuppressedError : function (error, suppressed, message) {
    var e = new Error(message);
    return e.name = "SuppressedError", e.error = error, e.suppressed = suppressed, e;
};

/******************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */
/* global Reflect, Promise, SuppressedError, Symbol */


function __classPrivateFieldGet(receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
}

function __classPrivateFieldSet(receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
}

typeof SuppressedError === "function" ? SuppressedError : function (error, suppressed, message) {
    var e = new Error(message);
    return e.name = "SuppressedError", e.error = error, e.suppressed = suppressed, e;
};

var _Cache_keyArray, _Cache_maxLength;
class Cache extends Map {
    constructor() {
        super(...arguments);
        _Cache_keyArray.set(this, []);
        _Cache_maxLength.set(this, 30);
    }
    setMaxLength(length) {
        __classPrivateFieldSet(this, _Cache_maxLength, length, "f");
        this._checkLength();
    }
    set(key, value) {
        if (this.has(key))
            return this;
        __classPrivateFieldGet(this, _Cache_keyArray, "f").push(key);
        this._checkLength();
        return super.set(key, value);
    }
    _checkLength() {
        while (__classPrivateFieldGet(this, _Cache_keyArray, "f").length > __classPrivateFieldGet(this, _Cache_maxLength, "f")) {
            const key = __classPrivateFieldGet(this, _Cache_keyArray, "f").shift();
            this.delete(key);
        }
    }
}
_Cache_keyArray = new WeakMap(), _Cache_maxLength = new WeakMap();

const lowlight = createLowlight(all);
lowlight.register("vue", function hljsDefineVue(hljs) {
    return {
        subLanguage: "xml",
        contains: [
            hljs.COMMENT("<!--", "-->", {
                relevance: 10,
            }),
            {
                begin: /^(\s*)(<script>)/gm,
                end: /^(\s*)(<\/script>)/gm,
                subLanguage: "javascript",
                excludeBegin: true,
                excludeEnd: true,
            },
            {
                begin: /^(?:\s*)(?:<script\s+lang=(["'])ts\1>)/gm,
                end: /^(\s*)(<\/script>)/gm,
                subLanguage: "typescript",
                excludeBegin: true,
                excludeEnd: true,
            },
            {
                begin: /^(\s*)(<style(\s+scoped)?>)/gm,
                end: /^(\s*)(<\/style>)/gm,
                subLanguage: "css",
                excludeBegin: true,
                excludeEnd: true,
            },
            {
                begin: /^(?:\s*)(?:<style(?:\s+scoped)?\s+lang=(["'])(?:s[ca]ss)\1(?:\s+scoped)?>)/gm,
                end: /^(\s*)(<\/style>)/gm,
                subLanguage: "scss",
                excludeBegin: true,
                excludeEnd: true,
            },
            {
                begin: /^(?:\s*)(?:<style(?:\s+scoped)?\s+lang=(["'])stylus\1(?:\s+scoped)?>)/gm,
                end: /^(\s*)(<\/style>)/gm,
                subLanguage: "stylus",
                excludeBegin: true,
                excludeEnd: true,
            },
        ],
    };
});
const highlighter = lowlight;
let _autoDetectLang = true;
let _maxLineToIgnoreSyntax = 2000;
const _ignoreSyntaxHighlightList = [];
Object.defineProperty(highlighter, "maxLineToIgnoreSyntax", {
    get: () => _maxLineToIgnoreSyntax,
});
Object.defineProperty(highlighter, "setMaxLineToIgnoreSyntax", {
    value: (v) => {
        _maxLineToIgnoreSyntax = v;
    },
});
Object.defineProperty(highlighter, "autoDetectLang", {
    get: () => _autoDetectLang,
});
Object.defineProperty(highlighter, "setAutoDetectLang", {
    value: (v) => {
        _autoDetectLang = v;
    },
});
Object.defineProperty(highlighter, "ignoreSyntaxHighlightList", {
    get: () => _ignoreSyntaxHighlightList,
});
Object.defineProperty(highlighter, "setIgnoreSyntaxHighlightList", {
    value: (v) => {
        _ignoreSyntaxHighlightList.length = 0;
        _ignoreSyntaxHighlightList.push(...v);
    },
});

var _File_instances, _File_doAST, _File_doCheck;
const map = new Cache();
map.setMaxLength(50);
map.name = "@git-diff-view/core";
if (process.env.NODE_ENV === "development" && typeof globalThis !== "undefined") {
    if (Array.isArray(globalThis.__diff_cache__)) {
        globalThis.__diff_cache__ = globalThis.__diff_cache__.filter((i) => i !== map);
        if (globalThis.__diff_cache__.length > 0) {
            console.warn("there are multiple instance of @git-diff-view/core in the one environment!");
        }
        globalThis.__diff_cache__.push(map);
    }
    else {
        globalThis.__diff_cache__ = [map];
    }
}
class File {
    constructor(raw, lang, fileName) {
        _File_instances.add(this);
        this.raw = raw;
        this.lang = lang;
        this.fileName = fileName;
        this.rawFile = {};
        this.hasDoRaw = false;
        this.syntaxFile = {};
        this.hasDoSyntax = false;
        this.maxLineNumber = 0;
        Object.defineProperty(this, "__v_skip", { value: true });
    }
    doSyntax({ autoDetectLang, registerHighlighter, }) {
        if (!this.raw || this.hasDoSyntax)
            return;
        let hasRegisteredLang = true;
        const _highlighter = registerHighlighter || highlighter;
        if (this.syntaxLength) {
            process.env.NODE_ENV === "development" && console.error("current file already doSyntax before!");
            return;
        }
        if (!_highlighter.registered(this.lang)) {
            hasRegisteredLang = false;
            if (!autoDetectLang) {
                process.env.NODE_ENV === "development" && console.warn(`not support current lang: ${this.lang} yet`);
                return;
            }
        }
        if (this.rawLength > _highlighter.maxLineToIgnoreSyntax) {
            process.env.NODE_ENV === "development" && console.warn(`ignore syntax for current file, because the rawLength is too long: ${this.rawLength}`);
            return;
        }
        if (this.fileName &&
            _highlighter.ignoreSyntaxHighlightList.some((item) => item instanceof RegExp ? item.test(this.fileName) : this.fileName === item)) {
            process.env.NODE_ENV === "development" &&
                console.warn(`ignore syntax for current file, because the fileName is in the ignoreSyntaxHighlightList: ${this.fileName}`);
            return;
        }
        if (hasRegisteredLang) {
            this.ast = _highlighter.highlight(this.lang, this.raw);
        }
        else {
            this.ast = _highlighter.highlightAuto(this.raw);
        }
        __classPrivateFieldGet(this, _File_instances, "m", _File_doAST).call(this);
        if (process.env.NODE_ENV === "development") {
            __classPrivateFieldGet(this, _File_instances, "m", _File_doCheck).call(this);
        }
        this.hasDoSyntax = true;
    }
    doRaw() {
        if (!this.raw || this.hasDoRaw)
            return;
        const rawString = this.raw;
        const rawArray = rawString.split("\n");
        this.rawLength = rawArray.length;
        this.maxLineNumber = rawArray.length;
        this.rawFile = {};
        for (let i = 0; i < rawArray.length; i++) {
            this.rawFile[i + 1] = i < rawArray.length - 1 ? rawArray[i] + "\n" : rawArray[i];
        }
        // reduce 对于大数组性能很差
        // this.rawFile = rawArray.reduce(
        //   (p, item, index) => ({
        //     ...p,
        //     [index + 1]: index < rawArray.length - 1 ? item + "\n" : item,
        //   }),
        //   {}
        // );
        this.hasDoRaw = true;
    }
}
_File_instances = new WeakSet(), _File_doAST = function _File_doAST() {
    const ast = this.ast;
    let lineNumber = 1;
    const syntaxObj = this.syntaxFile;
    const loopAST = (nodes, wrapper) => {
        nodes.forEach((node) => {
            if (node.type === "text") {
                if (node.value.indexOf("\n") === -1) {
                    const valueLength = node.value.length;
                    if (!syntaxObj[lineNumber]) {
                        node.startIndex = 0;
                        node.endIndex = valueLength - 1;
                        const item = {
                            value: node.value,
                            lineNumber,
                            valueLength,
                            nodeList: [{ node, wrapper }],
                        };
                        syntaxObj[lineNumber] = item;
                    }
                    else {
                        node.startIndex = syntaxObj[lineNumber].valueLength;
                        node.endIndex = node.startIndex + valueLength - 1;
                        syntaxObj[lineNumber].value += node.value;
                        syntaxObj[lineNumber].valueLength += valueLength;
                        syntaxObj[lineNumber].nodeList.push({ node, wrapper });
                    }
                    node.lineNumber = lineNumber;
                    return;
                }
                const lines = node.value.split("\n");
                node.children = node.children || [];
                for (let i = 0; i < lines.length; i++) {
                    const _value = i === lines.length - 1 ? lines[i] : lines[i] + "\n";
                    const _lineNumber = i === 0 ? lineNumber : ++lineNumber;
                    const _valueLength = _value.length;
                    const _node = {
                        type: "text",
                        value: _value,
                        startIndex: Infinity,
                        endIndex: Infinity,
                        lineNumber: _lineNumber,
                    };
                    if (!syntaxObj[_lineNumber]) {
                        _node.startIndex = 0;
                        _node.endIndex = _valueLength - 1;
                        const item = {
                            value: _value,
                            lineNumber: _lineNumber,
                            valueLength: _valueLength,
                            nodeList: [{ node: _node, wrapper }],
                        };
                        syntaxObj[_lineNumber] = item;
                    }
                    else {
                        _node.startIndex = syntaxObj[_lineNumber].valueLength;
                        _node.endIndex = _node.startIndex + _valueLength - 1;
                        syntaxObj[_lineNumber].value += _value;
                        syntaxObj[_lineNumber].valueLength += _valueLength;
                        syntaxObj[_lineNumber].nodeList.push({ node: _node, wrapper });
                    }
                    node.children.push(_node);
                }
                node.lineNumber = lineNumber;
                return;
            }
            if (node.children) {
                loopAST(node.children, node);
                node.lineNumber = lineNumber;
            }
        });
    };
    loopAST(ast.children);
    this.syntaxLength = lineNumber;
}, _File_doCheck = function _File_doCheck() {
    if (this.rawLength && this.syntaxLength) {
        if (this.rawLength !== this.syntaxLength) {
            console.warn("the rawLength not match for the syntaxLength");
        }
        Object.values(this.syntaxFile).forEach(({ value, lineNumber }) => {
            if (value !== this.rawFile[lineNumber]) {
                console.log("some line not match:" + value + " __ " + this.rawFile[lineNumber] + " __ at: " + lineNumber + " lineNumber");
            }
        });
    }
};
const getFile = (raw, lang, fileName) => {
    const key = raw + "--" + "0.0.7" + "--" + lang;
    if (map.has(key))
        return map.get(key);
    const file = new File(raw, lang, fileName);
    map.set(key, file);
    return file;
};
const _cacheMap = map;

const maxLength = 1000;
/** Get the maximum position in the range. */
function rangeMax(range) {
    return range.location + range.length;
}
/** Get the length of the common substring between the two strings. */
function commonLength(stringA, rangeA, stringB, rangeB, reverse) {
    const max = Math.min(rangeA.length, rangeB.length);
    const startA = reverse ? rangeMax(rangeA) - 1 : rangeA.location;
    const startB = reverse ? rangeMax(rangeB) - 1 : rangeB.location;
    const stride = reverse ? -1 : 1;
    let length = 0;
    while (Math.abs(length) < max) {
        if (stringA[startA + length] !== stringB[startB + length]) {
            break;
        }
        length += stride;
    }
    return Math.abs(length);
}
function isInValidString(s) {
    return s.trim().length === 0 || s.length >= maxLength;
}
/** Get the changed ranges in the strings, relative to each other. */
function relativeChanges(stringA, stringB) {
    let bRange = { location: 0, length: stringB.length };
    let aRange = { location: 0, length: stringA.length };
    const _stringA = stringA.trimEnd();
    const _stringB = stringB.trimEnd();
    const aEndStr = stringA.slice(-2);
    const bEndStr = stringB.slice(-2);
    if (_stringA === _stringB && aEndStr !== bEndStr && (aEndStr === "\r\n" || bEndStr === "\r\n")) {
        return {
            stringARange: {
                location: _stringA.length,
                length: stringA.length - _stringA.length,
                isNewLineSymbolChanged: true,
            },
            stringBRange: {
                location: _stringB.length,
                length: stringB.length - _stringB.length,
                isNewLineSymbolChanged: true,
            },
        };
    }
    if (isInValidString(stringA) || isInValidString(stringB)) {
        aRange.length = 0;
        bRange.length = 0;
        return { stringARange: aRange, stringBRange: bRange };
    }
    const prefixLength = commonLength(stringB, bRange, stringA, aRange, false);
    bRange = {
        location: bRange.location + prefixLength,
        length: bRange.length - prefixLength,
    };
    aRange = {
        location: aRange.location + prefixLength,
        length: aRange.length - prefixLength,
    };
    const suffixLength = commonLength(stringB, bRange, stringA, aRange, true);
    bRange.length -= suffixLength;
    aRange.length -= suffixLength;
    return { stringARange: aRange, stringBRange: bRange };
}
/** Check two string have a diff range */
function hasRelativeChange(stringA, stringB) {
    const _stringA = stringA.trim();
    const _stringB = stringB.trim();
    const { stringARange, stringBRange } = relativeChanges(_stringA, _stringB);
    return (stringARange.location > 0 ||
        stringBRange.location > 0 ||
        stringARange.length < _stringA.length ||
        stringBRange.length < _stringB.length);
}

/** indicate what a line in the diff represents */
var DiffLineType;
(function (DiffLineType) {
    DiffLineType[DiffLineType["Context"] = 0] = "Context";
    DiffLineType[DiffLineType["Add"] = 1] = "Add";
    DiffLineType[DiffLineType["Delete"] = 2] = "Delete";
    DiffLineType[DiffLineType["Hunk"] = 3] = "Hunk";
})(DiffLineType || (DiffLineType = {}));
/** track details related to each line in the diff */
class DiffLine {
    constructor(text, type, 
    // Line number in the original diff patch (before expanding it), or null if
    // it was added as part of a diff expansion action.
    originalLineNumber, oldLineNumber, newLineNumber, noTrailingNewLine = false, needRematch = false, range) {
        this.text = text;
        this.type = type;
        this.originalLineNumber = originalLineNumber;
        this.oldLineNumber = oldLineNumber;
        this.newLineNumber = newLineNumber;
        this.noTrailingNewLine = noTrailingNewLine;
        this.needRematch = needRematch;
        this.range = range;
    }
    withNoTrailingNewLine(noTrailingNewLine) {
        return new DiffLine(this.text, this.type, this.originalLineNumber, this.oldLineNumber, this.newLineNumber, noTrailingNewLine);
    }
    isIncludeableLine() {
        return this.type === DiffLineType.Add || this.type === DiffLineType.Delete;
    }
    /** The content of the line, i.e., without the line type marker. */
    get content() {
        return this.text.substring(1);
    }
    equals(other) {
        return (this.text === other.text &&
            this.type === other.type &&
            this.originalLineNumber === other.originalLineNumber &&
            this.oldLineNumber === other.oldLineNumber &&
            this.newLineNumber === other.newLineNumber &&
            this.noTrailingNewLine === other.noTrailingNewLine);
    }
}
const checkDiffLineIncludeChange = (diffLine) => {
    if (!diffLine)
        return false;
    return diffLine.type === DiffLineType.Add || diffLine.type === DiffLineType.Delete;
};

var DiffHunkExpansionType;
(function (DiffHunkExpansionType) {
    /** The hunk header cannot be expanded at all. */
    DiffHunkExpansionType["None"] = "None";
    /**
     * The hunk header can be expanded up exclusively. Only the first hunk can be
     * expanded up exclusively.
     */
    DiffHunkExpansionType["Up"] = "Up";
    /**
     * The hunk header can be expanded down exclusively. Only the last hunk (if
     * it's the dummy hunk with only one line) can be expanded down exclusively.
     */
    DiffHunkExpansionType["Down"] = "Down";
    /** The hunk header can be expanded both up and down. */
    DiffHunkExpansionType["Both"] = "Both";
    /**
     * The hunk header represents a short gap that, when expanded, will
     * result in merging this hunk and the hunk above.
     */
    DiffHunkExpansionType["Short"] = "Short";
})(DiffHunkExpansionType || (DiffHunkExpansionType = {}));
/** each diff is made up of a number of hunks */
class DiffHunk {
    /**
     * @param header The details from the diff hunk header about the line start and patch length.
     * @param lines The contents - context and changes - of the diff section.
     * @param unifiedDiffStart The diff hunk's start position in the overall file diff.
     * @param unifiedDiffEnd The diff hunk's end position in the overall file diff.
     */
    constructor(header, lines, unifiedDiffStart, unifiedDiffEnd, expansionType) {
        this.header = header;
        this.lines = lines;
        this.unifiedDiffStart = unifiedDiffStart;
        this.unifiedDiffEnd = unifiedDiffEnd;
        this.expansionType = expansionType;
    }
    equals(other) {
        if (this === other) {
            return true;
        }
        return (this.header.equals(other.header) &&
            this.unifiedDiffStart === other.unifiedDiffStart &&
            this.unifiedDiffEnd === other.unifiedDiffEnd &&
            this.expansionType === other.expansionType &&
            this.lines.length === other.lines.length &&
            this.lines.every((xLine, ix) => xLine.equals(other.lines[ix])));
    }
}
/** details about the start and end of a diff hunk */
class DiffHunkHeader {
    /**
     * @param oldStartLine The line in the old (or original) file where this diff hunk starts.
     * @param oldLineCount The number of lines in the old (or original) file that this diff hunk covers
     * @param newStartLine The line in the new file where this diff hunk starts.
     * @param newLineCount The number of lines in the new file that this diff hunk covers.
     */
    constructor(oldStartLine, oldLineCount, newStartLine, newLineCount) {
        this.oldStartLine = oldStartLine;
        this.oldLineCount = oldLineCount;
        this.newStartLine = newStartLine;
        this.newLineCount = newLineCount;
    }
    toDiffLineRepresentation() {
        return `@@ -${this.oldStartLine},${this.oldLineCount} +${this.newStartLine},${this.newLineCount} @@`;
    }
    equals(other) {
        return (this.oldStartLine === other.oldStartLine &&
            this.oldLineCount === other.oldLineCount &&
            this.newStartLine === other.newStartLine &&
            this.oldStartLine === other.oldStartLine);
    }
}

/** How many new lines will be added to a diff hunk by default. */
const DefaultDiffExpansionStep = 20;
function assertNever(_, message) {
    throw new Error(message);
}
/** Utility function for getting the digit count of the largest line number in an array of diff hunks */
function getLargestLineNumber(hunks) {
    var _a, _b;
    if (hunks.length === 0) {
        return 0;
    }
    for (let i = hunks.length - 1; i >= 0; i--) {
        const hunk = hunks[i];
        for (let j = hunk.lines.length - 1; j >= 0; j--) {
            const line = hunk.lines[j];
            if (line.type === DiffLineType.Hunk) {
                continue;
            }
            const newLineNumber = (_a = line.newLineNumber) !== null && _a !== void 0 ? _a : 0;
            const oldLineNumber = (_b = line.oldLineNumber) !== null && _b !== void 0 ? _b : 0;
            return newLineNumber > oldLineNumber ? newLineNumber : oldLineNumber;
        }
    }
    return 0;
}
/**
 * Calculates whether or not a hunk header can be expanded up, down, both, or if
 * the space represented by the hunk header is short and expansion there would
 * mean merging with the hunk above.
 *
 * @param hunkIndex     Index of the hunk to evaluate within the whole diff.
 * @param hunkHeader    Header of the hunk to evaluate.
 * @param previousHunk  Hunk previous to the one to evaluate. Null if the
 *                      evaluated hunk is the first one.
 */
function getHunkHeaderExpansionType(hunkIndex, hunkHeader, previousHunk) {
    const distanceToPrevious = previousHunk === null
        ? Infinity
        : hunkHeader.oldStartLine - previousHunk.header.oldStartLine - previousHunk.header.oldLineCount;
    // In order to simplify the whole logic around expansion, only the hunk at the
    // top can be expanded up exclusively, and only the hunk at the bottom (the
    // dummy one, see getTextDiffWithBottomDummyHunk) can be expanded down
    // exclusively.
    // The rest of the hunks can be expanded both ways, except those which are too
    // short and therefore the direction of expansion doesn't matter.
    if (hunkIndex === 0) {
        // The top hunk can only be expanded if there is content above it
        if (hunkHeader.oldStartLine > 1 && hunkHeader.newStartLine > 1) {
            return DiffHunkExpansionType.Up;
        }
        else {
            return DiffHunkExpansionType.None;
        }
    }
    else if (distanceToPrevious <= DefaultDiffExpansionStep) {
        return DiffHunkExpansionType.Short;
    }
    else {
        return DiffHunkExpansionType.Both;
    }
}
const numIterator = (num, cb) => {
    const re = [];
    for (let i = 0; i < num; i++) {
        re.push(cb(i));
    }
    return re;
};
const getLang = (fileName) => {
    const dotIndex = fileName.lastIndexOf(".");
    const extension = fileName.slice(dotIndex + 1);
    return extension;
};
const getDiffRange = (additions, deletions) => {
    if (additions.length === deletions.length) {
        const len = additions.length;
        for (let i = 0; i < len; i++) {
            const addition = additions[i];
            const deletion = deletions[i];
            const hasDiffRange = hasRelativeChange(addition.text, deletion.text);
            if (hasDiffRange) {
                const { stringARange, stringBRange } = relativeChanges(addition.text, deletion.text);
                addition.needRematch = true;
                addition.range = stringARange;
                deletion.needRematch = true;
                deletion.range = stringBRange;
            }
        }
    }
};

/* eslint-disable max-lines */
// NODE: ALL of the Diff parse logic from desktop, SEE https://github.com/desktop/desktop
// https://en.wikipedia.org/wiki/Diff_utility
//
// @@ -l,s +l,s @@ optional section heading
//
// The hunk range information contains two hunk ranges. The range for the hunk of the original
// file is preceded by a minus symbol, and the range for the new file is preceded by a plus
// symbol. Each hunk range is of the format l,s where l is the starting line number and s is
// the number of lines the change hunk applies to for each respective file.
//
// In many versions of GNU diff, each range can omit the comma and trailing value s,
// in which case s defaults to 1
const diffHeaderRe = /^@@ -(\d+)(?:,(\d+))? \+(\d+)(?:,(\d+))? @@/;
/**
 * Regular expression matching invisible bidirectional Unicode characters that
 * may be interpreted or compiled differently than what it appears. More info:
 * https://github.co/hiddenchars
 */
const HiddenBidiCharsRegex = /[\u202A-\u202E]|[\u2066-\u2069]/;
const DiffPrefixAdd = "+";
const DiffPrefixDelete = "-";
const DiffPrefixContext = " ";
const DiffPrefixNoNewline = "\\";
const DiffLinePrefixChars = new Set([
    DiffPrefixAdd,
    DiffPrefixDelete,
    DiffPrefixContext,
    DiffPrefixNoNewline,
]);
/**
 * A parser for the GNU unified diff format
 *
 * See https://www.gnu.org/software/diffutils/manual/html_node/Detailed-Unified.html
 */
class DiffParser {
    constructor() {
        Object.defineProperty(this, "__v_skip", { value: true });
        this.reset();
    }
    /**
     * Resets the internal parser state so that it can be reused.
     *
     * This is done automatically at the end of each parse run.
     */
    reset() {
        this.ls = 0;
        this.le = -1;
        this.text = "";
    }
    /**
     * Aligns the internal character pointers at the boundaries of
     * the next line.
     *
     * Returns true if successful or false if the end of the diff
     * has been reached.
     */
    nextLine() {
        this.ls = this.le + 1;
        // We've reached the end of the diff
        if (this.ls >= this.text.length) {
            return false;
        }
        this.le = this.text.indexOf("\n", this.ls);
        // If we can't find the next newline character we'll put our
        // end pointer at the end of the diff string
        if (this.le === -1) {
            this.le = this.text.length;
        }
        // We've succeeded if there's anything to read in between the
        // start and the end
        return this.ls !== this.le;
    }
    /**
     * Advances to the next line and returns it as a substring
     * of the raw diff text. Returns null if end of diff was
     * reached.
     */
    readLine(header) {
        if (header) {
            return this.nextLine() ? this.text.substring(this.ls, this.le) : null;
        }
        else {
            return this.nextLine() ? this.text.substring(this.ls + 1, this.le + 1) : null;
        }
    }
    /** Tests if the current line starts with the given search text */
    lineStartsWith(searchString) {
        return this.text.startsWith(searchString, this.ls);
    }
    /** Tests if the current line ends with the given search text */
    lineEndsWith(searchString) {
        return this.text.endsWith(searchString, this.le);
    }
    /**
     * Returns the starting character of the next line without
     * advancing the internal state. Returns null if advancing
     * would mean reaching the end of the diff.
     */
    peek() {
        const p = this.le + 1;
        return p < this.text.length ? this.text[p] : null;
    }
    /**
     * Parse the diff header, meaning everything from the
     * start of the diff output to the end of the line beginning
     * with +++
     *
     * Example diff header:
     *
     *   diff --git a/app/src/lib/diff-parser.ts b/app/src/lib/diff-parser.ts
     *   index e1d4871..3bd3ee0 100644
     *   --- a/app/src/lib/diff-parser.ts
     *   +++ b/app/src/lib/diff-parser.ts
     *
     * Returns an object with information extracted from the diff
     * header (currently whether it's a binary patch) or null if
     * the end of the diff was reached before the +++ line could be
     * found (which is a valid state).
     */
    parseDiffHeader() {
        // TODO: There's information in here that we might want to
        // capture, such as mode changes
        while (this.nextLine()) {
            if (this.lineStartsWith("Binary files ") && this.lineEndsWith("differ")) {
                return { isBinary: true };
            }
            if (this.lineStartsWith("+++")) {
                return { isBinary: false };
            }
        }
        // It's not an error to not find the +++ line, see the
        // 'parses diff of empty file' test in diff-parser-tests.ts
        return null;
    }
    /**
     * Attempts to convert a RegExp capture group into a number.
     * If the group doesn't exist or wasn't captured the function
     * will return the value of the defaultValue parameter or throw
     * an error if no default value was provided. If the captured
     * string can't be converted to a number an error will be thrown.
     */
    numberFromGroup(m, group, defaultValue = null) {
        const str = m[group];
        if (!str) {
            if (!defaultValue) {
                throw new Error(`Group ${group} missing from regexp match and no defaultValue was provided`);
            }
            return defaultValue;
        }
        const num = parseInt(str, 10);
        if (isNaN(num)) {
            throw new Error(`Could not parse capture group ${group} into number: ${str}`);
        }
        return num;
    }
    /**
     * Parses a hunk header or throws an error if the given line isn't
     * a well-formed hunk header.
     *
     * We currently only extract the line number information and
     * ignore any hunk headings.
     *
     * Example hunk header (text within ``):
     *
     * `@@ -84,10 +82,8 @@ export function parseRawDiff(lines: ReadonlyArray<string>): Diff {`
     *
     * Where everything after the last @@ is what's known as the hunk, or section, heading
     */
    parseHunkHeader(line) {
        const m = diffHeaderRe.exec(line);
        if (!m) {
            throw new Error(`Invalid hunk header format`);
        }
        // If endLines are missing default to 1, see diffHeaderRe docs
        const oldStartLine = this.numberFromGroup(m, 1);
        const oldLineCount = this.numberFromGroup(m, 2, 1);
        const newStartLine = this.numberFromGroup(m, 3);
        const newLineCount = this.numberFromGroup(m, 4, 1);
        return new DiffHunkHeader(oldStartLine, oldLineCount, newStartLine, newLineCount);
    }
    /**
     * Convenience function which lets us leverage the type system to
     * prove exhaustive checks in parseHunk.
     *
     * Takes an arbitrary string and checks to see if the first character
     * of that string is one of the allowed prefix characters for diff
     * lines (ie lines in between hunk headers).
     */
    parseLinePrefix(c) {
        // Since we know that DiffLinePrefixChars and the DiffLinePrefix type
        // include the same characters we can tell the type system that we
        // now know that c[0] is one of the characters in the DifflinePrefix set
        if (c && c.length && DiffLinePrefixChars.has(c[0])) {
            return c[0];
        }
        return null;
    }
    /**
     * Parses a hunk, including its header or throws an error if the diff doesn't
     * contain a well-formed diff hunk at the current position.
     *
     * Expects that the position has been advanced to the beginning of a presumed
     * diff hunk header.
     *
     * @param linesConsumed The number of unified diff lines consumed up until
     *                      this point by the diff parser. Used to give the
     *                      position and length (in lines) of the parsed hunk
     *                      relative to the overall parsed diff. These numbers
     *                      have no real meaning in the context of a diff and
     *                      are only used to aid the app in line-selections.
     */
    parseHunk(linesConsumed, hunkIndex, previousHunk) {
        const headerLine = this.readLine(true);
        if (!headerLine) {
            throw new Error("Expected hunk header but reached end of diff");
        }
        const header = this.parseHunkHeader(headerLine);
        const lines = new Array();
        lines.push(new DiffLine(headerLine, DiffLineType.Hunk, 1, null, null));
        let c;
        let rollingDiffBeforeCounter = header.oldStartLine;
        let rollingDiffAfterCounter = header.newStartLine;
        let diffLineNumber = linesConsumed;
        while ((c = this.parseLinePrefix(this.peek()))) {
            const line = this.readLine(false);
            diffLineNumber++;
            if (!line) {
                throw new Error("Expected unified diff line but reached end of diff");
            }
            // A marker indicating that the last line in the original or the new file
            // is missing a trailing newline. In other words, the presence of this marker
            // means that the new and/or original file lacks a trailing newline.
            //
            // When we find it we have to look up the previous line and set the
            // noTrailingNewLine flag
            if (c === DiffPrefixNoNewline) {
                // See https://github.com/git/git/blob/21f862b498925194f8f1ebe8203b7a7df756555b/apply.c#L1725-L1732
                if (line.length < 12) {
                    throw new Error(`Expected "no newline at end of file" marker to be at least 12 bytes long`);
                }
                const previousLineIndex = lines.length - 1;
                const previousLine = lines[previousLineIndex];
                lines[previousLineIndex] = previousLine.withNoTrailingNewLine(true);
                continue;
            }
            let diffLine;
            if (c === DiffPrefixAdd) {
                diffLine = new DiffLine(line, DiffLineType.Add, diffLineNumber, null, rollingDiffAfterCounter++);
            }
            else if (c === DiffPrefixDelete) {
                diffLine = new DiffLine(line, DiffLineType.Delete, diffLineNumber, rollingDiffBeforeCounter++, null);
            }
            else if (c === DiffPrefixContext) {
                diffLine = new DiffLine(line, DiffLineType.Context, diffLineNumber, rollingDiffBeforeCounter++, rollingDiffAfterCounter++);
            }
            else {
                return assertNever(c, `Unknown DiffLinePrefix: ${c}`);
            }
            lines.push(diffLine);
        }
        if (lines.length === 1) {
            throw new Error("Malformed diff, empty hunk");
        }
        return new DiffHunk(header, lines, linesConsumed, linesConsumed + lines.length - 1, getHunkHeaderExpansionType(hunkIndex, header, previousHunk));
    }
    /**
     * Parse a well-formed unified diff into hunks and lines.
     *
     * @param text A unified diff produced by git diff, git log --patch
     *             or any other git plumbing command that produces unified
     *             diffs.
     */
    parse(text) {
        this.text = text;
        try {
            const headerInfo = this.parseDiffHeader();
            const headerEnd = this.le;
            const header = this.text.substring(0, headerEnd);
            // empty diff
            if (!headerInfo) {
                return {
                    header,
                    contents: "",
                    hunks: [],
                    isBinary: false,
                    maxLineNumber: 0,
                    hasHiddenBidiChars: false,
                };
            }
            if (headerInfo.isBinary) {
                return {
                    header,
                    contents: "",
                    hunks: [],
                    isBinary: true,
                    maxLineNumber: 0,
                    hasHiddenBidiChars: false,
                };
            }
            const hunks = new Array();
            let linesConsumed = 0;
            let previousHunk = null;
            do {
                const hunk = this.parseHunk(linesConsumed, hunks.length, previousHunk);
                hunks.push(hunk);
                previousHunk = hunk;
                linesConsumed += hunk.lines.length;
            } while (this.peek());
            const contents = this.text
                .substring(headerEnd + 1, this.le)
                // Note that this simply returns a reference to the
                // substring if no match is found, it does not create
                // a new string instance.
                .replace(/\n\\ No newline at end of file/g, "");
            return {
                header,
                contents,
                hunks,
                isBinary: headerInfo.isBinary,
                maxLineNumber: getLargestLineNumber(hunks),
                hasHiddenBidiChars: HiddenBidiCharsRegex.test(text),
            };
        }
        finally {
            this.reset();
        }
    }
}
const parseInstance = new DiffParser();

var _DiffFile_instances, _DiffFile_oldFileResult, _DiffFile_newFileResult, _DiffFile_diffListResults, _DiffFile_diffLines, _DiffFile_oldFileDiffLines, _DiffFile_newFileDiffLines, _DiffFile_oldFileLines, _DiffFile_newFileLines, _DiffFile_oldFileSyntaxLines, _DiffFile_newFileSyntaxLines, _DiffFile_oldFilePlaceholderLines, _DiffFile_newFilePlaceholderLines, _DiffFile_splitLeftLines, _DiffFile_splitRightLines, _DiffFile_splitHunksLines, _DiffFile_unifiedLines, _DiffFile_unifiedHunksLines, _DiffFile_listeners, _DiffFile_hasInitRaw, _DiffFile_hasInitSyntax, _DiffFile_hasBuildSplit, _DiffFile_hasBuildUnified, _DiffFile_updateCount, _DiffFile_composeByDiff, _DiffFile_id, _DiffFile_clonedInstance, _DiffFile_doDiff, _DiffFile_doFile, _DiffFile_composeRaw, _DiffFile_composeFile, _DiffFile_composeDiff, _DiffFile_composeSyntax, _DiffFile_getOldDiffLine, _DiffFile_getNewDiffLine, _DiffFile_getOldRawLine, _DiffFile_getNewRawLine;
const composeLen = 40;
const idSet = new Set();
class DiffFile {
    static createInstance(data, bundle) {
        var _a, _b, _c, _d, _e, _f;
        const instance = new DiffFile(((_a = data === null || data === void 0 ? void 0 : data.oldFile) === null || _a === void 0 ? void 0 : _a.fileName) || "", ((_b = data === null || data === void 0 ? void 0 : data.oldFile) === null || _b === void 0 ? void 0 : _b.content) || "", ((_c = data === null || data === void 0 ? void 0 : data.newFile) === null || _c === void 0 ? void 0 : _c.fileName) || "", ((_d = data === null || data === void 0 ? void 0 : data.newFile) === null || _d === void 0 ? void 0 : _d.content) || "", (data === null || data === void 0 ? void 0 : data.hunks) || [], ((_e = data === null || data === void 0 ? void 0 : data.oldFile) === null || _e === void 0 ? void 0 : _e.fileLang) || "", ((_f = data === null || data === void 0 ? void 0 : data.newFile) === null || _f === void 0 ? void 0 : _f.fileLang) || "");
        if (bundle) {
            instance.mergeBundle(bundle);
        }
        return instance;
    }
    constructor(_oldFileName, _oldFileContent, _newFileName, _newFileContent, _diffList, _oldFileLang, _newFileLang) {
        _DiffFile_instances.add(this);
        this._oldFileName = _oldFileName;
        this._newFileName = _newFileName;
        this._diffList = _diffList;
        _DiffFile_oldFileResult.set(this, void 0);
        _DiffFile_newFileResult.set(this, void 0);
        _DiffFile_diffListResults.set(this, void 0);
        _DiffFile_diffLines.set(this, void 0);
        _DiffFile_oldFileDiffLines.set(this, void 0);
        _DiffFile_newFileDiffLines.set(this, void 0);
        _DiffFile_oldFileLines.set(this, void 0);
        _DiffFile_newFileLines.set(this, void 0);
        _DiffFile_oldFileSyntaxLines.set(this, void 0);
        _DiffFile_newFileSyntaxLines.set(this, void 0);
        _DiffFile_oldFilePlaceholderLines.set(this, void 0);
        _DiffFile_newFilePlaceholderLines.set(this, void 0);
        _DiffFile_splitLeftLines.set(this, []);
        _DiffFile_splitRightLines.set(this, []);
        _DiffFile_splitHunksLines.set(this, void 0);
        _DiffFile_unifiedLines.set(this, []);
        _DiffFile_unifiedHunksLines.set(this, void 0);
        _DiffFile_listeners.set(this, []);
        _DiffFile_hasInitRaw.set(this, false);
        _DiffFile_hasInitSyntax.set(this, false);
        _DiffFile_hasBuildSplit.set(this, false);
        _DiffFile_hasBuildUnified.set(this, false);
        _DiffFile_updateCount.set(this, 0);
        _DiffFile_composeByDiff.set(this, false);
        this._version_ = "0.0.7";
        this._oldFileContent = "";
        this._oldFileLang = "";
        this._newFileContent = "";
        this._newFileLang = "";
        this.diffLineLength = 0;
        this.splitLineLength = 0;
        this.unifiedLineLength = 0;
        _DiffFile_id.set(this, "");
        _DiffFile_clonedInstance.set(this, new Map());
        this.getSplitLeftLine = (index) => {
            return __classPrivateFieldGet(this, _DiffFile_splitLeftLines, "f")[index];
        };
        this.getSplitRightLine = (index) => {
            return __classPrivateFieldGet(this, _DiffFile_splitRightLines, "f")[index];
        };
        this.getSplitHunkLine = (index) => {
            var _a;
            return (_a = __classPrivateFieldGet(this, _DiffFile_splitHunksLines, "f")) === null || _a === void 0 ? void 0 : _a[index];
        };
        this.onSplitHunkExpand = (dir, index, needTrigger = true) => {
            var _a, _b;
            const current = (_a = __classPrivateFieldGet(this, _DiffFile_splitHunksLines, "f")) === null || _a === void 0 ? void 0 : _a[index];
            if (!current || !current.splitInfo)
                return;
            if (__classPrivateFieldGet(this, _DiffFile_composeByDiff, "f"))
                return;
            if (dir === "all") {
                for (let i = current.splitInfo.startHiddenIndex; i < current.splitInfo.endHiddenIndex; i++) {
                    const leftLine = __classPrivateFieldGet(this, _DiffFile_splitLeftLines, "f")[i];
                    const rightLine = __classPrivateFieldGet(this, _DiffFile_splitRightLines, "f")[i];
                    if (leftLine === null || leftLine === void 0 ? void 0 : leftLine.isHidden)
                        leftLine.isHidden = false;
                    if (rightLine === null || rightLine === void 0 ? void 0 : rightLine.isHidden)
                        rightLine.isHidden = false;
                }
                current.splitInfo = Object.assign(Object.assign(Object.assign({}, current.splitInfo), current.hunkInfo), { plainText: current.text, startHiddenIndex: current.splitInfo.endHiddenIndex });
            }
            else if (dir === "down") {
                for (let i = current.splitInfo.startHiddenIndex; i < current.splitInfo.startHiddenIndex + composeLen; i++) {
                    const leftLine = __classPrivateFieldGet(this, _DiffFile_splitLeftLines, "f")[i];
                    const rightLine = __classPrivateFieldGet(this, _DiffFile_splitRightLines, "f")[i];
                    if (leftLine === null || leftLine === void 0 ? void 0 : leftLine.isHidden)
                        leftLine.isHidden = false;
                    if (rightLine === null || rightLine === void 0 ? void 0 : rightLine.isHidden)
                        rightLine.isHidden = false;
                }
                if (current.isLast) {
                    current.splitInfo = Object.assign(Object.assign({}, current.splitInfo), { startHiddenIndex: current.splitInfo.startHiddenIndex + composeLen });
                }
                else {
                    current.splitInfo = Object.assign(Object.assign({}, current.splitInfo), { startHiddenIndex: current.splitInfo.startHiddenIndex + composeLen, plainText: `@@ -${current.splitInfo.oldStartIndex},${current.splitInfo.oldLength} +${current.splitInfo.newStartIndex},${current.splitInfo.newLength}` });
                }
            }
            else {
                if (current.isLast) {
                    process.env.NODE_ENV === "development" && console.error("the last hunk can not expand up!");
                    return;
                }
                for (let i = current.splitInfo.endHiddenIndex - composeLen; i < current.splitInfo.endHiddenIndex; i++) {
                    const leftLine = __classPrivateFieldGet(this, _DiffFile_splitLeftLines, "f")[i];
                    const rightLine = __classPrivateFieldGet(this, _DiffFile_splitRightLines, "f")[i];
                    if (leftLine === null || leftLine === void 0 ? void 0 : leftLine.isHidden)
                        leftLine.isHidden = false;
                    if (rightLine === null || rightLine === void 0 ? void 0 : rightLine.isHidden)
                        rightLine.isHidden = false;
                }
                const oldStartIndex = current.splitInfo.oldStartIndex - composeLen;
                const oldLength = current.splitInfo.oldLength + composeLen;
                const newStartIndex = current.splitInfo.newStartIndex - composeLen;
                const newLength = current.splitInfo.newLength + composeLen;
                current.splitInfo = Object.assign(Object.assign({}, current.splitInfo), { endHiddenIndex: current.splitInfo.endHiddenIndex - composeLen, oldStartIndex,
                    oldLength,
                    newStartIndex,
                    newLength, plainText: `@@ -${oldStartIndex},${oldLength} +${newStartIndex},${newLength}` });
                (_b = __classPrivateFieldGet(this, _DiffFile_splitHunksLines, "f")) === null || _b === void 0 ? true : delete _b[index];
                __classPrivateFieldGet(this, _DiffFile_splitHunksLines, "f")[current.splitInfo.endHiddenIndex] = current;
            }
            needTrigger && this.notifyAll();
        };
        this.getUnifiedLine = (index) => {
            return __classPrivateFieldGet(this, _DiffFile_unifiedLines, "f")[index];
        };
        this.getUnifiedHunkLine = (index) => {
            var _a;
            return (_a = __classPrivateFieldGet(this, _DiffFile_unifiedHunksLines, "f")) === null || _a === void 0 ? void 0 : _a[index];
        };
        this.onUnifiedHunkExpand = (dir, index, needTrigger = true) => {
            var _a, _b, _c;
            const current = (_a = __classPrivateFieldGet(this, _DiffFile_unifiedHunksLines, "f")) === null || _a === void 0 ? void 0 : _a[index];
            if (!current || !current.unifiedInfo)
                return;
            if (__classPrivateFieldGet(this, _DiffFile_composeByDiff, "f"))
                return;
            if (dir === "all") {
                for (let i = current.unifiedInfo.startHiddenIndex; i < current.unifiedInfo.endHiddenIndex; i++) {
                    const unifiedLine = (_b = __classPrivateFieldGet(this, _DiffFile_unifiedLines, "f")) === null || _b === void 0 ? void 0 : _b[i];
                    if (unifiedLine === null || unifiedLine === void 0 ? void 0 : unifiedLine.isHidden) {
                        unifiedLine.isHidden = false;
                    }
                }
                current.unifiedInfo = Object.assign(Object.assign(Object.assign({}, current.unifiedInfo), current.hunkInfo), { plainText: current.text, startHiddenIndex: current.unifiedInfo.endHiddenIndex });
            }
            else if (dir === "down") {
                for (let i = current.unifiedInfo.startHiddenIndex; i < current.unifiedInfo.startHiddenIndex + composeLen; i++) {
                    const unifiedLine = __classPrivateFieldGet(this, _DiffFile_unifiedLines, "f")[i];
                    if (unifiedLine === null || unifiedLine === void 0 ? void 0 : unifiedLine.isHidden)
                        unifiedLine.isHidden = false;
                }
                if (current.isLast) {
                    current.unifiedInfo = Object.assign(Object.assign({}, current.unifiedInfo), { startHiddenIndex: current.unifiedInfo.startHiddenIndex + composeLen });
                }
                else {
                    current.unifiedInfo = Object.assign(Object.assign({}, current.unifiedInfo), { startHiddenIndex: current.unifiedInfo.startHiddenIndex + composeLen, plainText: `@@ -${current.unifiedInfo.oldStartIndex},${current.unifiedInfo.oldLength} +${current.unifiedInfo.newStartIndex},${current.unifiedInfo.newLength}` });
                }
            }
            else {
                if (current.isLast) {
                    process.env.NODE_ENV === "development" && console.error("the last hunk can not expand up!");
                    return;
                }
                for (let i = current.unifiedInfo.endHiddenIndex - composeLen; i < current.unifiedInfo.endHiddenIndex; i++) {
                    const unifiedLine = __classPrivateFieldGet(this, _DiffFile_unifiedLines, "f")[i];
                    if (unifiedLine === null || unifiedLine === void 0 ? void 0 : unifiedLine.isHidden)
                        unifiedLine.isHidden = false;
                }
                const oldStartIndex = current.unifiedInfo.oldStartIndex - composeLen;
                const oldLength = current.unifiedInfo.oldLength + composeLen;
                const newStartIndex = current.unifiedInfo.newStartIndex - composeLen;
                const newLength = current.unifiedInfo.newLength + composeLen;
                current.unifiedInfo = Object.assign(Object.assign({}, current.unifiedInfo), { endHiddenIndex: current.unifiedInfo.endHiddenIndex - composeLen, oldStartIndex,
                    oldLength,
                    newStartIndex,
                    newLength, plainText: `@@ -${oldStartIndex},${oldLength} +${newStartIndex},${newLength}` });
                (_c = __classPrivateFieldGet(this, _DiffFile_unifiedHunksLines, "f")) === null || _c === void 0 ? true : delete _c[index];
                __classPrivateFieldGet(this, _DiffFile_unifiedHunksLines, "f")[current.unifiedInfo.endHiddenIndex] = current;
            }
            needTrigger && this.notifyAll();
        };
        this.onAllExpand = (mode) => {
            if (__classPrivateFieldGet(this, _DiffFile_composeByDiff, "f"))
                return;
            if (mode === "split") {
                Object.keys(__classPrivateFieldGet(this, _DiffFile_splitHunksLines, "f") || {}).forEach((key) => {
                    this.onSplitHunkExpand("all", +key, false);
                });
            }
            else {
                Object.keys(__classPrivateFieldGet(this, _DiffFile_unifiedHunksLines, "f") || {}).forEach((key) => {
                    this.onUnifiedHunkExpand("all", +key, false);
                });
            }
            this.notifyAll();
        };
        this.onAllCollapse = (mode) => {
            if (__classPrivateFieldGet(this, _DiffFile_composeByDiff, "f"))
                return;
            if (mode === "split") {
                Object.values(__classPrivateFieldGet(this, _DiffFile_splitLeftLines, "f") || {}).forEach((item) => {
                    if (!item.isHidden && item._isHidden) {
                        item.isHidden = item._isHidden;
                    }
                });
                Object.values(__classPrivateFieldGet(this, _DiffFile_splitRightLines, "f") || {}).forEach((item) => {
                    if (!item.isHidden && item._isHidden) {
                        item.isHidden = item._isHidden;
                    }
                });
                Object.values(__classPrivateFieldGet(this, _DiffFile_splitHunksLines, "f") || {}).forEach((item) => {
                    if (!item.splitInfo)
                        return;
                    item.splitInfo = Object.assign(Object.assign({}, item.splitInfo), { oldStartIndex: item.splitInfo._oldStartIndex, oldLength: item.splitInfo._oldLength, newStartIndex: item.splitInfo._newStartIndex, newLength: item.splitInfo._newLength, startHiddenIndex: item.splitInfo._startHiddenIndex, endHiddenIndex: item.splitInfo._endHiddenIndex, plainText: item.splitInfo._plainText });
                });
                Object.keys(__classPrivateFieldGet(this, _DiffFile_splitHunksLines, "f") || {}).forEach((key) => {
                    const item = __classPrivateFieldGet(this, _DiffFile_splitHunksLines, "f")[key];
                    if (!item.splitInfo)
                        return;
                    if (item.splitInfo.endHiddenIndex !== +key) {
                        delete __classPrivateFieldGet(this, _DiffFile_splitHunksLines, "f")[key];
                        __classPrivateFieldGet(this, _DiffFile_splitHunksLines, "f")[item.splitInfo.endHiddenIndex] = item;
                    }
                });
            }
            else {
                Object.values(__classPrivateFieldGet(this, _DiffFile_unifiedLines, "f") || {}).forEach((item) => {
                    if (!item.isHidden && item._isHidden) {
                        item.isHidden = item._isHidden;
                    }
                });
                Object.values(__classPrivateFieldGet(this, _DiffFile_unifiedHunksLines, "f") || {}).forEach((item) => {
                    if (!item.unifiedInfo)
                        return;
                    item.unifiedInfo = Object.assign(Object.assign({}, item.unifiedInfo), { oldStartIndex: item.unifiedInfo._oldStartIndex, oldLength: item.unifiedInfo._oldLength, newStartIndex: item.unifiedInfo._newStartIndex, newLength: item.unifiedInfo._newLength, startHiddenIndex: item.unifiedInfo._startHiddenIndex, endHiddenIndex: item.unifiedInfo._endHiddenIndex, plainText: item.unifiedInfo._plainText });
                });
                Object.keys(__classPrivateFieldGet(this, _DiffFile_unifiedHunksLines, "f") || {}).forEach((key) => {
                    const item = __classPrivateFieldGet(this, _DiffFile_unifiedHunksLines, "f")[key];
                    if (!item.unifiedInfo)
                        return;
                    if (item.unifiedInfo.endHiddenIndex !== +key) {
                        delete __classPrivateFieldGet(this, _DiffFile_unifiedHunksLines, "f")[key];
                        __classPrivateFieldGet(this, _DiffFile_unifiedHunksLines, "f")[item.unifiedInfo.endHiddenIndex] = item;
                    }
                });
            }
            this.notifyAll();
        };
        this.getOldSyntaxLine = (lineNumber) => {
            var _a;
            return (_a = __classPrivateFieldGet(this, _DiffFile_oldFileSyntaxLines, "f")) === null || _a === void 0 ? void 0 : _a[lineNumber];
        };
        this.getNewSyntaxLine = (lineNumber) => {
            var _a;
            return (_a = __classPrivateFieldGet(this, _DiffFile_newFileSyntaxLines, "f")) === null || _a === void 0 ? void 0 : _a[lineNumber];
        };
        this.subscribe = (listener) => {
            __classPrivateFieldGet(this, _DiffFile_listeners, "f").push(listener);
            return () => {
                __classPrivateFieldGet(this, _DiffFile_listeners, "f").filter((i) => i !== listener);
            };
        };
        this.notifyAll = (skipSyncExternal) => {
            var _a;
            __classPrivateFieldSet(this, _DiffFile_updateCount, (_a = __classPrivateFieldGet(this, _DiffFile_updateCount, "f"), _a++, _a), "f");
            __classPrivateFieldGet(this, _DiffFile_listeners, "f").forEach((f) => {
                if (skipSyncExternal && f.isSyncExternal) {
                    return;
                }
                f();
            });
        };
        this.getUpdateCount = () => __classPrivateFieldGet(this, _DiffFile_updateCount, "f");
        this.getExpandEnabled = () => !__classPrivateFieldGet(this, _DiffFile_composeByDiff, "f");
        this.getBundle = () => {
            // common
            const hasInitRaw = __classPrivateFieldGet(this, _DiffFile_hasInitRaw, "f");
            const hasInitSyntax = __classPrivateFieldGet(this, _DiffFile_hasInitSyntax, "f");
            const hasBuildSplit = __classPrivateFieldGet(this, _DiffFile_hasBuildSplit, "f");
            const hasBuildUnified = __classPrivateFieldGet(this, _DiffFile_hasBuildUnified, "f");
            const oldFileLines = __classPrivateFieldGet(this, _DiffFile_oldFileLines, "f");
            const oldFileDiffLines = __classPrivateFieldGet(this, _DiffFile_oldFileDiffLines, "f");
            const oldFileSyntaxLines = __classPrivateFieldGet(this, _DiffFile_oldFileSyntaxLines, "f");
            const oldFilePlaceholderLines = __classPrivateFieldGet(this, _DiffFile_oldFilePlaceholderLines, "f");
            const newFileLines = __classPrivateFieldGet(this, _DiffFile_newFileLines, "f");
            const newFileDiffLines = __classPrivateFieldGet(this, _DiffFile_newFileDiffLines, "f");
            const newFileSyntaxLines = __classPrivateFieldGet(this, _DiffFile_newFileSyntaxLines, "f");
            const newFilePlaceholderLines = __classPrivateFieldGet(this, _DiffFile_newFilePlaceholderLines, "f");
            const splitLineLength = this.splitLineLength;
            const unifiedLineLength = this.unifiedLineLength;
            const composeByDiff = __classPrivateFieldGet(this, _DiffFile_composeByDiff, "f");
            // split
            const splitLeftLines = __classPrivateFieldGet(this, _DiffFile_splitLeftLines, "f");
            const splitRightLines = __classPrivateFieldGet(this, _DiffFile_splitRightLines, "f");
            const splitHunkLines = __classPrivateFieldGet(this, _DiffFile_splitHunksLines, "f");
            // unified
            const unifiedLines = __classPrivateFieldGet(this, _DiffFile_unifiedLines, "f");
            const unifiedHunkLines = __classPrivateFieldGet(this, _DiffFile_unifiedHunksLines, "f");
            const version = this._version_;
            return {
                hasInitRaw,
                hasInitSyntax,
                hasBuildSplit,
                hasBuildUnified,
                oldFileLines,
                oldFileDiffLines,
                oldFileSyntaxLines,
                oldFilePlaceholderLines,
                newFileLines,
                newFileDiffLines,
                newFileSyntaxLines,
                newFilePlaceholderLines,
                splitLineLength,
                unifiedLineLength,
                splitLeftLines,
                splitRightLines,
                splitHunkLines,
                unifiedLines,
                unifiedHunkLines,
                composeByDiff,
                version,
            };
        };
        this.mergeBundle = (data) => {
            __classPrivateFieldSet(this, _DiffFile_hasInitRaw, data.hasInitRaw, "f");
            __classPrivateFieldSet(this, _DiffFile_hasInitSyntax, data.hasInitSyntax, "f");
            __classPrivateFieldSet(this, _DiffFile_hasBuildSplit, data.hasBuildSplit, "f");
            __classPrivateFieldSet(this, _DiffFile_hasBuildUnified, data.hasBuildUnified, "f");
            __classPrivateFieldSet(this, _DiffFile_composeByDiff, data.composeByDiff, "f");
            __classPrivateFieldSet(this, _DiffFile_oldFileLines, data.oldFileLines, "f");
            __classPrivateFieldSet(this, _DiffFile_oldFileDiffLines, data.oldFileDiffLines, "f");
            __classPrivateFieldSet(this, _DiffFile_oldFileSyntaxLines, data.oldFileSyntaxLines, "f");
            __classPrivateFieldSet(this, _DiffFile_oldFilePlaceholderLines, data.oldFilePlaceholderLines, "f");
            __classPrivateFieldSet(this, _DiffFile_newFileLines, data.newFileLines, "f");
            __classPrivateFieldSet(this, _DiffFile_newFileDiffLines, data.newFileDiffLines, "f");
            __classPrivateFieldSet(this, _DiffFile_newFileSyntaxLines, data.newFileSyntaxLines, "f");
            __classPrivateFieldSet(this, _DiffFile_newFilePlaceholderLines, data.newFilePlaceholderLines, "f");
            this.splitLineLength = data.splitLineLength;
            this.unifiedLineLength = data.unifiedLineLength;
            __classPrivateFieldSet(this, _DiffFile_splitLeftLines, data.splitLeftLines, "f");
            __classPrivateFieldSet(this, _DiffFile_splitRightLines, data.splitRightLines, "f");
            __classPrivateFieldSet(this, _DiffFile_splitHunksLines, data.splitHunkLines, "f");
            __classPrivateFieldSet(this, _DiffFile_unifiedLines, data.unifiedLines, "f");
            __classPrivateFieldSet(this, _DiffFile_unifiedHunksLines, data.unifiedHunkLines, "f");
            if (process.env.NODE_ENV === "development" && this._version_ !== data.version) {
                console.error("the version of the `diffInstance` is not match, some error may happen!");
            }
            this.notifyAll();
        };
        this._getIsPureDiffRender = () => __classPrivateFieldGet(this, _DiffFile_composeByDiff, "f");
        this._addClonedInstance = (instance) => {
            const updateFunc = () => {
                this._notifyOthers(instance);
            };
            updateFunc.isSyncExternal = true;
            const unsubscribe = instance.subscribe(updateFunc);
            __classPrivateFieldGet(this, _DiffFile_clonedInstance, "f").set(instance, unsubscribe);
        };
        this._notifyOthers = (instance) => {
            __classPrivateFieldGet(this, _DiffFile_clonedInstance, "f").forEach((_, i) => {
                if (i !== instance) {
                    i.notifyAll(true);
                }
            });
        };
        this._delClonedInstance = (instance) => {
            const unsubscribe = __classPrivateFieldGet(this, _DiffFile_clonedInstance, "f").get(instance);
            unsubscribe && unsubscribe();
            __classPrivateFieldGet(this, _DiffFile_clonedInstance, "f").delete(instance);
        };
        this._getFullBundle = () => {
            const bundle = this.getBundle();
            const oldFileResult = __classPrivateFieldGet(this, _DiffFile_oldFileResult, "f");
            const newFileResult = __classPrivateFieldGet(this, _DiffFile_newFileResult, "f");
            const diffLines = __classPrivateFieldGet(this, _DiffFile_diffLines, "f");
            const diffListResults = __classPrivateFieldGet(this, _DiffFile_diffListResults, "f");
            return Object.assign(Object.assign({}, bundle), { oldFileResult,
                newFileResult,
                diffLines,
                diffListResults });
        };
        this._mergeFullBundle = (data) => {
            this.mergeBundle(data);
            __classPrivateFieldSet(this, _DiffFile_oldFileResult, data.oldFileResult, "f");
            __classPrivateFieldSet(this, _DiffFile_newFileResult, data.newFileResult, "f");
            __classPrivateFieldSet(this, _DiffFile_diffLines, data.diffLines, "f");
            __classPrivateFieldSet(this, _DiffFile_diffListResults, data.diffListResults, "f");
        };
        this._destroy = () => {
            this.clearId();
            __classPrivateFieldGet(this, _DiffFile_listeners, "f").splice(0, __classPrivateFieldGet(this, _DiffFile_listeners, "f").length);
            __classPrivateFieldGet(this, _DiffFile_clonedInstance, "f").forEach((v) => v());
            __classPrivateFieldGet(this, _DiffFile_clonedInstance, "f").clear();
        };
        this.clear = () => {
            this._destroy();
            __classPrivateFieldSet(this, _DiffFile_oldFileResult, null, "f");
            __classPrivateFieldSet(this, _DiffFile_newFileResult, null, "f");
            __classPrivateFieldSet(this, _DiffFile_diffLines, null, "f");
            __classPrivateFieldSet(this, _DiffFile_diffListResults, null, "f");
            __classPrivateFieldSet(this, _DiffFile_newFileDiffLines, null, "f");
            __classPrivateFieldSet(this, _DiffFile_oldFileDiffLines, null, "f");
            __classPrivateFieldSet(this, _DiffFile_newFileLines, null, "f");
            __classPrivateFieldSet(this, _DiffFile_oldFileLines, null, "f");
            __classPrivateFieldSet(this, _DiffFile_newFileSyntaxLines, null, "f");
            __classPrivateFieldSet(this, _DiffFile_oldFileSyntaxLines, null, "f");
            __classPrivateFieldSet(this, _DiffFile_splitHunksLines, null, "f");
            __classPrivateFieldSet(this, _DiffFile_splitLeftLines, null, "f");
            __classPrivateFieldSet(this, _DiffFile_splitRightLines, null, "f");
            __classPrivateFieldSet(this, _DiffFile_unifiedHunksLines, null, "f");
            __classPrivateFieldSet(this, _DiffFile_unifiedLines, null, "f");
        };
        Object.defineProperty(this, "__v_skip", { value: true });
        let oldContent = _oldFileContent;
        let newContent = _newFileContent;
        const diffList = Array.from(new Set(_diffList));
        Object.defineProperties(this, {
            _oldFileName: { get: () => _oldFileName },
            _newFileName: { get: () => _newFileName },
            _oldFileLang: { get: () => getLang(_oldFileLang || _oldFileName || _newFileLang || _newFileName) || "txt" },
            _newFileLang: { get: () => getLang(_newFileLang || _newFileName || _oldFileLang || _oldFileName) || "txt" },
            _oldFileContent: {
                get: () => oldContent,
                set: (v) => (oldContent = v),
            },
            _newFileContent: {
                get: () => newContent,
                set: (v) => (newContent = v),
            },
            _diffList: { get: () => diffList },
        });
        this.initId();
    }
    initId() {
        let id = "--" + Math.random().toString().slice(2);
        while (idSet.has(id)) {
            id = "--" + Math.random().toString().slice(2);
        }
        idSet.add(id);
        __classPrivateFieldSet(this, _DiffFile_id, id, "f");
    }
    getId() {
        return __classPrivateFieldGet(this, _DiffFile_id, "f");
    }
    clearId() {
        idSet.delete(__classPrivateFieldGet(this, _DiffFile_id, "f"));
    }
    initRaw() {
        if (__classPrivateFieldGet(this, _DiffFile_hasInitRaw, "f"))
            return;
        __classPrivateFieldGet(this, _DiffFile_instances, "m", _DiffFile_doDiff).call(this);
        __classPrivateFieldGet(this, _DiffFile_instances, "m", _DiffFile_composeDiff).call(this);
        __classPrivateFieldGet(this, _DiffFile_instances, "m", _DiffFile_doFile).call(this);
        __classPrivateFieldGet(this, _DiffFile_instances, "m", _DiffFile_composeRaw).call(this);
        __classPrivateFieldGet(this, _DiffFile_instances, "m", _DiffFile_composeFile).call(this);
        __classPrivateFieldSet(this, _DiffFile_hasInitRaw, true, "f");
    }
    initSyntax({ autoDetectLang, registerHighlighter, } = {}) {
        if (__classPrivateFieldGet(this, _DiffFile_hasInitSyntax, "f"))
            return;
        __classPrivateFieldGet(this, _DiffFile_instances, "m", _DiffFile_composeSyntax).call(this, { registerHighlighter, autoDetectLang });
        __classPrivateFieldSet(this, _DiffFile_hasInitSyntax, true, "f");
    }
    init() {
        this.initRaw();
        this.initSyntax();
    }
    buildSplitDiffLines() {
        var _a, _b, _c, _d, _e, _f, _g;
        if (__classPrivateFieldGet(this, _DiffFile_hasBuildSplit, "f"))
            return;
        let oldFileLineNumber = 1;
        let newFileLineNumber = 1;
        let prevIsHidden = false;
        let hideStart = Infinity;
        const maxOldFileLineNumber = ((_a = __classPrivateFieldGet(this, _DiffFile_oldFileResult, "f")) === null || _a === void 0 ? void 0 : _a.maxLineNumber) || 0;
        const maxNewFileLineNumber = ((_b = __classPrivateFieldGet(this, _DiffFile_newFileResult, "f")) === null || _b === void 0 ? void 0 : _b.maxLineNumber) || 0;
        while (oldFileLineNumber <= maxOldFileLineNumber || newFileLineNumber <= maxNewFileLineNumber) {
            const oldDiffLine = __classPrivateFieldGet(this, _DiffFile_instances, "m", _DiffFile_getOldDiffLine).call(this, oldFileLineNumber);
            const newDiffLine = __classPrivateFieldGet(this, _DiffFile_instances, "m", _DiffFile_getNewDiffLine).call(this, newFileLineNumber);
            const oldRawLine = __classPrivateFieldGet(this, _DiffFile_instances, "m", _DiffFile_getOldRawLine).call(this, oldFileLineNumber);
            const newRawLine = __classPrivateFieldGet(this, _DiffFile_instances, "m", _DiffFile_getNewRawLine).call(this, newFileLineNumber);
            const oldLineHasChange = oldDiffLine === null || oldDiffLine === void 0 ? void 0 : oldDiffLine.isIncludeableLine();
            const newLineHasChange = newDiffLine === null || newDiffLine === void 0 ? void 0 : newDiffLine.isIncludeableLine();
            const len = __classPrivateFieldGet(this, _DiffFile_splitRightLines, "f").length;
            const isHidden = !oldDiffLine && !newDiffLine;
            if (oldDiffLine && !newDiffLine) {
                if (oldDiffLine.newLineNumber && oldDiffLine.newLineNumber > newFileLineNumber) {
                    newFileLineNumber++;
                    continue;
                }
                if (oldDiffLine.newLineNumber === null || oldDiffLine.newLineNumber === undefined) {
                    newFileLineNumber++;
                }
            }
            if (newDiffLine && !oldDiffLine) {
                if (newDiffLine.oldLineNumber && newDiffLine.oldLineNumber > oldFileLineNumber) {
                    oldFileLineNumber++;
                    continue;
                }
                if (newDiffLine.oldLineNumber === null || newDiffLine.oldLineNumber === undefined) {
                    oldFileLineNumber++;
                }
            }
            if (!oldDiffLine && !oldRawLine && !newDiffLine && !newRawLine)
                break;
            if (!oldDiffLine && !newDiffLine) {
                if (((_c = __classPrivateFieldGet(this, _DiffFile_oldFilePlaceholderLines, "f")) === null || _c === void 0 ? void 0 : _c[oldFileLineNumber]) && ((_d = __classPrivateFieldGet(this, _DiffFile_newFilePlaceholderLines, "f")) === null || _d === void 0 ? void 0 : _d[newFileLineNumber])) {
                    oldFileLineNumber++;
                    newFileLineNumber++;
                    continue;
                }
                if (!oldRawLine && ((_e = __classPrivateFieldGet(this, _DiffFile_newFilePlaceholderLines, "f")) === null || _e === void 0 ? void 0 : _e[newFileLineNumber])) {
                    newFileLineNumber++;
                    continue;
                }
                if (!newRawLine && ((_f = __classPrivateFieldGet(this, _DiffFile_oldFilePlaceholderLines, "f")) === null || _f === void 0 ? void 0 : _f[oldFileLineNumber])) {
                    oldFileLineNumber++;
                    continue;
                }
            }
            if ((oldLineHasChange && newLineHasChange) || (!oldLineHasChange && !newLineHasChange)) {
                __classPrivateFieldGet(this, _DiffFile_splitLeftLines, "f").push({
                    lineNumber: oldFileLineNumber++,
                    value: oldRawLine,
                    diff: oldDiffLine,
                    isHidden,
                    _isHidden: isHidden,
                });
                __classPrivateFieldGet(this, _DiffFile_splitRightLines, "f").push({
                    lineNumber: newFileLineNumber++,
                    value: newRawLine,
                    diff: newDiffLine,
                    isHidden,
                    _isHidden: isHidden,
                });
            }
            else if (oldLineHasChange) {
                __classPrivateFieldGet(this, _DiffFile_splitLeftLines, "f").push({
                    lineNumber: oldFileLineNumber++,
                    value: oldRawLine,
                    diff: oldDiffLine,
                    isHidden,
                    _isHidden: isHidden,
                });
                __classPrivateFieldGet(this, _DiffFile_splitRightLines, "f").push({});
            }
            else if (newLineHasChange) {
                __classPrivateFieldGet(this, _DiffFile_splitLeftLines, "f").push({});
                __classPrivateFieldGet(this, _DiffFile_splitRightLines, "f").push({
                    lineNumber: newFileLineNumber++,
                    value: newRawLine,
                    diff: newDiffLine,
                    isHidden,
                    _isHidden: isHidden,
                });
            }
            if (!prevIsHidden && isHidden) {
                hideStart = len;
            }
            prevIsHidden = isHidden;
            if (oldDiffLine && newDiffLine && !oldLineHasChange && !newLineHasChange) {
                const current = newDiffLine;
                const previous = newDiffLine.index ? (_g = __classPrivateFieldGet(this, _DiffFile_diffLines, "f")) === null || _g === void 0 ? void 0 : _g[current.index - 1] : undefined;
                if (previous && previous.type === DiffLineType.Hunk) {
                    const typedPrevious = previous;
                    if (Number.isFinite(hideStart)) {
                        typedPrevious.splitInfo = Object.assign(Object.assign({}, typedPrevious.hunkInfo), { startHiddenIndex: hideStart, endHiddenIndex: len, plainText: typedPrevious.text, _startHiddenIndex: hideStart, _endHiddenIndex: len, _plainText: typedPrevious.text });
                        hideStart = Infinity;
                    }
                    __classPrivateFieldSet(this, _DiffFile_splitHunksLines, Object.assign(Object.assign({}, __classPrivateFieldGet(this, _DiffFile_splitHunksLines, "f")), { [len]: typedPrevious }), "f");
                }
            }
        }
        // have last hunk
        if (Number.isFinite(hideStart)) {
            const lastDiff = new DiffLine("", DiffLineType.Hunk, null, null, null);
            const lastHunk = lastDiff;
            lastHunk.isLast = true;
            lastHunk.splitInfo = {
                startHiddenIndex: hideStart,
                endHiddenIndex: __classPrivateFieldGet(this, _DiffFile_splitRightLines, "f").length,
                _startHiddenIndex: hideStart,
                _endHiddenIndex: __classPrivateFieldGet(this, _DiffFile_splitRightLines, "f").length,
                // just for placeholder
                plainText: "",
                oldStartIndex: 0,
                newStartIndex: 0,
                oldLength: 0,
                newLength: 0,
                _plainText: "",
                _oldStartIndex: 0,
                _newStartIndex: 0,
                _oldLength: 0,
                _newLength: 0,
            };
            __classPrivateFieldSet(this, _DiffFile_splitHunksLines, Object.assign(Object.assign({}, __classPrivateFieldGet(this, _DiffFile_splitHunksLines, "f")), { [__classPrivateFieldGet(this, _DiffFile_splitRightLines, "f").length]: lastHunk }), "f");
            hideStart = Infinity;
        }
        this.splitLineLength = __classPrivateFieldGet(this, _DiffFile_splitRightLines, "f").length;
        __classPrivateFieldSet(this, _DiffFile_hasBuildSplit, true, "f");
        this.notifyAll();
    }
    buildUnifiedDiffLines() {
        var _a, _b, _c, _d, _e, _f, _g;
        if (__classPrivateFieldGet(this, _DiffFile_hasBuildUnified, "f"))
            return;
        let oldFileLineNumber = 1;
        let newFileLineNumber = 1;
        let prevIsHidden = false;
        let hideStart = Infinity;
        const maxOldFileLineNumber = ((_a = __classPrivateFieldGet(this, _DiffFile_oldFileResult, "f")) === null || _a === void 0 ? void 0 : _a.maxLineNumber) || 0;
        const maxNewFileLineNumber = ((_b = __classPrivateFieldGet(this, _DiffFile_newFileResult, "f")) === null || _b === void 0 ? void 0 : _b.maxLineNumber) || 0;
        while (oldFileLineNumber <= maxOldFileLineNumber || newFileLineNumber <= maxNewFileLineNumber) {
            const oldRawLine = __classPrivateFieldGet(this, _DiffFile_instances, "m", _DiffFile_getOldRawLine).call(this, oldFileLineNumber);
            const oldDiffLine = __classPrivateFieldGet(this, _DiffFile_instances, "m", _DiffFile_getOldDiffLine).call(this, oldFileLineNumber);
            const newRawLine = __classPrivateFieldGet(this, _DiffFile_instances, "m", _DiffFile_getNewRawLine).call(this, newFileLineNumber);
            const newDiffLine = __classPrivateFieldGet(this, _DiffFile_instances, "m", _DiffFile_getNewDiffLine).call(this, newFileLineNumber);
            const oldLineHasChange = oldDiffLine === null || oldDiffLine === void 0 ? void 0 : oldDiffLine.isIncludeableLine();
            const newLineHasChange = newDiffLine === null || newDiffLine === void 0 ? void 0 : newDiffLine.isIncludeableLine();
            const len = __classPrivateFieldGet(this, _DiffFile_unifiedLines, "f").length;
            const isHidden = !oldDiffLine && !newDiffLine;
            if (oldDiffLine && !newDiffLine) {
                if (oldDiffLine.newLineNumber && oldDiffLine.newLineNumber > newFileLineNumber) {
                    newFileLineNumber++;
                    continue;
                }
                if (oldDiffLine.newLineNumber === null || oldDiffLine.newLineNumber === undefined) {
                    newFileLineNumber++;
                }
            }
            if (newDiffLine && !oldDiffLine) {
                if (newDiffLine.oldLineNumber && newDiffLine.oldLineNumber > oldFileLineNumber) {
                    oldFileLineNumber++;
                    continue;
                }
                if (newDiffLine.oldLineNumber === null || newDiffLine.oldLineNumber === undefined) {
                    oldFileLineNumber++;
                }
            }
            if (!oldRawLine && !newRawLine && !newDiffLine && !oldDiffLine)
                break;
            if (!oldDiffLine && !newDiffLine) {
                if (((_c = __classPrivateFieldGet(this, _DiffFile_oldFilePlaceholderLines, "f")) === null || _c === void 0 ? void 0 : _c[oldFileLineNumber]) && ((_d = __classPrivateFieldGet(this, _DiffFile_newFilePlaceholderLines, "f")) === null || _d === void 0 ? void 0 : _d[newFileLineNumber])) {
                    oldFileLineNumber++;
                    newFileLineNumber++;
                    continue;
                }
                if (!oldRawLine && ((_e = __classPrivateFieldGet(this, _DiffFile_newFilePlaceholderLines, "f")) === null || _e === void 0 ? void 0 : _e[newFileLineNumber])) {
                    newFileLineNumber++;
                    continue;
                }
                if (!newRawLine && ((_f = __classPrivateFieldGet(this, _DiffFile_oldFilePlaceholderLines, "f")) === null || _f === void 0 ? void 0 : _f[oldFileLineNumber])) {
                    oldFileLineNumber++;
                    continue;
                }
            }
            if (!oldLineHasChange && !newLineHasChange) {
                __classPrivateFieldGet(this, _DiffFile_unifiedLines, "f").push({
                    oldLineNumber: oldFileLineNumber++,
                    newLineNumber: newFileLineNumber++,
                    value: newRawLine,
                    diff: newDiffLine,
                    isHidden,
                    _isHidden: isHidden,
                });
            }
            else if (oldLineHasChange) {
                __classPrivateFieldGet(this, _DiffFile_unifiedLines, "f").push({
                    oldLineNumber: oldFileLineNumber++,
                    value: oldRawLine,
                    diff: oldDiffLine,
                    isHidden,
                    _isHidden: isHidden,
                });
            }
            else if (newLineHasChange) {
                __classPrivateFieldGet(this, _DiffFile_unifiedLines, "f").push({
                    newLineNumber: newFileLineNumber++,
                    value: newRawLine,
                    diff: newDiffLine,
                    isHidden,
                    _isHidden: isHidden,
                });
            }
            if (!prevIsHidden && isHidden) {
                hideStart = len;
            }
            prevIsHidden = isHidden;
            if (oldDiffLine && newDiffLine && !oldLineHasChange && !newLineHasChange) {
                const current = newDiffLine;
                const previous = current.index ? (_g = __classPrivateFieldGet(this, _DiffFile_diffLines, "f")) === null || _g === void 0 ? void 0 : _g[current.index - 1] : undefined;
                if (previous && previous.type === DiffLineType.Hunk) {
                    const typedPrevious = previous;
                    if (Number.isFinite(hideStart)) {
                        typedPrevious.unifiedInfo = Object.assign(Object.assign({}, typedPrevious.hunkInfo), { startHiddenIndex: hideStart, endHiddenIndex: len, plainText: typedPrevious.text, _startHiddenIndex: hideStart, _endHiddenIndex: len, _plainText: typedPrevious.text });
                        hideStart = Infinity;
                    }
                    __classPrivateFieldSet(this, _DiffFile_unifiedHunksLines, Object.assign(Object.assign({}, __classPrivateFieldGet(this, _DiffFile_unifiedHunksLines, "f")), { [len]: typedPrevious }), "f");
                }
            }
        }
        // have last hunk
        if (Number.isFinite(hideStart)) {
            const lastDiff = new DiffLine("", DiffLineType.Hunk, null, null, null);
            const lastHunk = lastDiff;
            lastHunk.isLast = true;
            lastHunk.unifiedInfo = {
                startHiddenIndex: hideStart,
                endHiddenIndex: __classPrivateFieldGet(this, _DiffFile_unifiedLines, "f").length,
                _startHiddenIndex: hideStart,
                _endHiddenIndex: __classPrivateFieldGet(this, _DiffFile_unifiedLines, "f").length,
                // just for placeholder
                plainText: "",
                oldStartIndex: 0,
                newStartIndex: 0,
                oldLength: 0,
                newLength: 0,
                _plainText: "",
                _oldStartIndex: 0,
                _newStartIndex: 0,
                _oldLength: 0,
                _newLength: 0,
            };
            __classPrivateFieldSet(this, _DiffFile_unifiedHunksLines, Object.assign(Object.assign({}, __classPrivateFieldGet(this, _DiffFile_unifiedHunksLines, "f")), { [__classPrivateFieldGet(this, _DiffFile_unifiedLines, "f").length]: lastHunk }), "f");
            hideStart = Infinity;
        }
        this.unifiedLineLength = __classPrivateFieldGet(this, _DiffFile_unifiedLines, "f").length;
        __classPrivateFieldSet(this, _DiffFile_hasBuildUnified, true, "f");
        this.notifyAll();
    }
}
_DiffFile_oldFileResult = new WeakMap(), _DiffFile_newFileResult = new WeakMap(), _DiffFile_diffListResults = new WeakMap(), _DiffFile_diffLines = new WeakMap(), _DiffFile_oldFileDiffLines = new WeakMap(), _DiffFile_newFileDiffLines = new WeakMap(), _DiffFile_oldFileLines = new WeakMap(), _DiffFile_newFileLines = new WeakMap(), _DiffFile_oldFileSyntaxLines = new WeakMap(), _DiffFile_newFileSyntaxLines = new WeakMap(), _DiffFile_oldFilePlaceholderLines = new WeakMap(), _DiffFile_newFilePlaceholderLines = new WeakMap(), _DiffFile_splitLeftLines = new WeakMap(), _DiffFile_splitRightLines = new WeakMap(), _DiffFile_splitHunksLines = new WeakMap(), _DiffFile_unifiedLines = new WeakMap(), _DiffFile_unifiedHunksLines = new WeakMap(), _DiffFile_listeners = new WeakMap(), _DiffFile_hasInitRaw = new WeakMap(), _DiffFile_hasInitSyntax = new WeakMap(), _DiffFile_hasBuildSplit = new WeakMap(), _DiffFile_hasBuildUnified = new WeakMap(), _DiffFile_updateCount = new WeakMap(), _DiffFile_composeByDiff = new WeakMap(), _DiffFile_id = new WeakMap(), _DiffFile_clonedInstance = new WeakMap(), _DiffFile_instances = new WeakSet(), _DiffFile_doDiff = function _DiffFile_doDiff() {
    if (!this._diffList)
        return;
    __classPrivateFieldSet(this, _DiffFile_diffListResults, this._diffList.map((s) => parseInstance.parse(s)), "f");
}, _DiffFile_doFile = function _DiffFile_doFile() {
    if (!this._oldFileContent && !this._newFileContent)
        return;
    if (this._oldFileContent) {
        __classPrivateFieldSet(this, _DiffFile_oldFileResult, getFile(this._oldFileContent, this._oldFileLang, this._oldFileName), "f");
    }
    if (this._newFileContent) {
        __classPrivateFieldSet(this, _DiffFile_newFileResult, getFile(this._newFileContent, this._newFileLang, this._newFileName), "f");
    }
}, _DiffFile_composeRaw = function _DiffFile_composeRaw() {
    var _a, _b, _c, _d;
    (_a = __classPrivateFieldGet(this, _DiffFile_oldFileResult, "f")) === null || _a === void 0 ? void 0 : _a.doRaw();
    __classPrivateFieldSet(this, _DiffFile_oldFileLines, (_b = __classPrivateFieldGet(this, _DiffFile_oldFileResult, "f")) === null || _b === void 0 ? void 0 : _b.rawFile, "f");
    (_c = __classPrivateFieldGet(this, _DiffFile_newFileResult, "f")) === null || _c === void 0 ? void 0 : _c.doRaw();
    __classPrivateFieldSet(this, _DiffFile_newFileLines, (_d = __classPrivateFieldGet(this, _DiffFile_newFileResult, "f")) === null || _d === void 0 ? void 0 : _d.rawFile, "f");
}, _DiffFile_composeFile = function _DiffFile_composeFile() {
    if (this._oldFileContent && this._newFileContent)
        return;
    const oldFilePlaceholderLines = {};
    const newFilePlaceholderLines = {};
    // all of the file content not exist, try to use diff result to compose
    if (!this._oldFileContent && !this._newFileContent) {
        let newLineNumber = 1;
        let oldLineNumber = 1;
        let oldFileContent = "";
        let newFileContent = "";
        while (oldLineNumber <= this.diffLineLength) {
            const index = oldLineNumber++;
            const diffLine = __classPrivateFieldGet(this, _DiffFile_instances, "m", _DiffFile_getOldDiffLine).call(this, index);
            if (diffLine) {
                oldFileContent += diffLine.text;
            }
            else {
                // empty line for placeholder
                oldFileContent += "\n";
                oldFilePlaceholderLines[index] = true;
            }
        }
        while (newLineNumber <= this.diffLineLength) {
            const index = newLineNumber++;
            const diffLine = __classPrivateFieldGet(this, _DiffFile_instances, "m", _DiffFile_getNewDiffLine).call(this, index);
            if (diffLine) {
                newFileContent += diffLine.text;
            }
            else {
                // empty line for placeholder
                newFileContent += "\n";
                newFilePlaceholderLines[index] = true;
            }
        }
        if (oldFileContent === newFileContent)
            return;
        this._oldFileContent = oldFileContent;
        this._newFileContent = newFileContent;
        __classPrivateFieldSet(this, _DiffFile_oldFileResult, getFile(this._oldFileContent, this._oldFileLang, this._oldFileName), "f");
        __classPrivateFieldSet(this, _DiffFile_newFileResult, getFile(this._newFileContent, this._newFileLang, this._newFileName), "f");
        __classPrivateFieldSet(this, _DiffFile_oldFilePlaceholderLines, oldFilePlaceholderLines, "f");
        __classPrivateFieldSet(this, _DiffFile_newFilePlaceholderLines, newFilePlaceholderLines, "f");
        // all of the file just compose by diff, so we can not do the expand action
        __classPrivateFieldSet(this, _DiffFile_composeByDiff, true, "f");
    }
    else if (__classPrivateFieldGet(this, _DiffFile_oldFileResult, "f")) {
        let newLineNumber = 1;
        let oldLineNumber = 1;
        let newFileContent = "";
        while (oldLineNumber <= __classPrivateFieldGet(this, _DiffFile_oldFileResult, "f").maxLineNumber) {
            const newDiffLine = __classPrivateFieldGet(this, _DiffFile_instances, "m", _DiffFile_getNewDiffLine).call(this, newLineNumber++);
            if (newDiffLine) {
                newFileContent += newDiffLine.text;
                oldLineNumber = newDiffLine.oldLineNumber ? newDiffLine.oldLineNumber + 1 : oldLineNumber;
            }
            else {
                newFileContent += __classPrivateFieldGet(this, _DiffFile_instances, "m", _DiffFile_getOldRawLine).call(this, oldLineNumber++);
            }
        }
        if (newFileContent === this._oldFileContent)
            return;
        this._newFileContent = newFileContent;
        __classPrivateFieldSet(this, _DiffFile_newFileResult, getFile(this._newFileContent, this._newFileLang, this._newFileName), "f");
    }
    else if (__classPrivateFieldGet(this, _DiffFile_newFileResult, "f")) {
        let oldLineNumber = 1;
        let newLineNumber = 1;
        let oldFileContent = "";
        while (newLineNumber <= __classPrivateFieldGet(this, _DiffFile_newFileResult, "f").maxLineNumber) {
            const oldDiffLine = __classPrivateFieldGet(this, _DiffFile_instances, "m", _DiffFile_getOldDiffLine).call(this, oldLineNumber++);
            if (oldDiffLine) {
                oldFileContent += oldDiffLine.text;
                newLineNumber = oldDiffLine.newLineNumber ? oldDiffLine.newLineNumber + 1 : newLineNumber;
            }
            else {
                oldFileContent += __classPrivateFieldGet(this, _DiffFile_instances, "m", _DiffFile_getNewRawLine).call(this, newLineNumber++);
            }
        }
        if (oldFileContent === this._newFileContent)
            return;
        this._oldFileContent = oldFileContent;
        __classPrivateFieldSet(this, _DiffFile_oldFileResult, getFile(this._oldFileContent, this._oldFileLang, this._oldFileName), "f");
    }
    __classPrivateFieldGet(this, _DiffFile_instances, "m", _DiffFile_composeRaw).call(this);
}, _DiffFile_composeDiff = function _DiffFile_composeDiff() {
    var _a;
    if (!((_a = __classPrivateFieldGet(this, _DiffFile_diffListResults, "f")) === null || _a === void 0 ? void 0 : _a.length))
        return;
    __classPrivateFieldGet(this, _DiffFile_diffListResults, "f").forEach((item) => {
        const hunks = item.hunks;
        hunks.forEach((hunk) => {
            let additions = [];
            let deletions = [];
            hunk.lines.forEach((line) => {
                if (line.type === DiffLineType.Add) {
                    additions.push(line);
                }
                else if (line.type === DiffLineType.Delete) {
                    deletions.push(line);
                }
                else {
                    getDiffRange(additions, deletions);
                    additions = [];
                    deletions = [];
                }
            });
            getDiffRange(additions, deletions);
        });
    });
    __classPrivateFieldSet(this, _DiffFile_diffLines, [], "f");
    const tmp = [];
    __classPrivateFieldGet(this, _DiffFile_diffListResults, "f").forEach((item) => {
        item.hunks.forEach((_item) => {
            tmp.push(..._item.lines);
        });
    });
    __classPrivateFieldSet(this, _DiffFile_diffLines, tmp.map((i, index) => {
        var _a;
        const typedI = i;
        typedI.index = index;
        if (typedI.type === DiffLineType.Hunk) {
            const numInfo = (_a = typedI.text.split("@@")) === null || _a === void 0 ? void 0 : _a[1].split(" ").filter(Boolean);
            const oldNumInfo = (numInfo === null || numInfo === void 0 ? void 0 : numInfo[0]) || "";
            const newNumInfo = (numInfo === null || numInfo === void 0 ? void 0 : numInfo[1]) || "";
            const [oldNumStartIndex, oldNumLength] = oldNumInfo.split(",");
            const [newNumStartIndex, newNumLength] = newNumInfo.split(",");
            const typedTypeI = typedI;
            typedTypeI.hunkInfo = {
                oldStartIndex: -Number(oldNumStartIndex),
                oldLength: Number(oldNumLength),
                newStartIndex: +Number(newNumStartIndex),
                newLength: Number(newNumLength),
                _oldStartIndex: -Number(oldNumStartIndex),
                _oldLength: Number(oldNumLength),
                _newStartIndex: +Number(newNumStartIndex),
                _newLength: Number(newNumLength),
            };
        }
        return typedI;
    }), "f");
    // this.#diffLines = this.#diffListResults
    //   .reduce<DiffLine[]>((p, c) => p.concat(...c.hunks.reduce<DiffLine[]>((_p, _c) => _p.concat(..._c.lines), [])), [])
    //   .map<DiffLineItem>((i, index) => {
    //     const typedI = i as DiffLineItem;
    //     typedI.index = index;
    //     if (typedI.type === DiffLineType.Hunk) {
    //       const numInfo = typedI.text.split("@@")?.[1].split(" ").filter(Boolean);
    //       const oldNumInfo = numInfo?.[0] || "";
    //       const newNumInfo = numInfo?.[1] || "";
    //       const [oldNumStartIndex, oldNumLength] = oldNumInfo.split(",");
    //       const [newNumStartIndex, newNumLength] = newNumInfo.split(",");
    //       const typedTypeI = typedI as DiffHunkItem;
    //       typedTypeI.hunkInfo = {
    //         oldStartIndex: -Number(oldNumStartIndex),
    //         oldLength: Number(oldNumLength),
    //         newStartIndex: +Number(newNumStartIndex),
    //         newLength: Number(newNumLength),
    //       };
    //     }
    //     return typedI;
    //   });
    __classPrivateFieldSet(this, _DiffFile_oldFileDiffLines, {}, "f");
    __classPrivateFieldGet(this, _DiffFile_diffLines, "f").forEach((item) => {
        if (item.oldLineNumber) {
            this.diffLineLength = Math.max(this.diffLineLength, item.oldLineNumber);
            __classPrivateFieldGet(this, _DiffFile_oldFileDiffLines, "f")[item.oldLineNumber] = item;
        }
    });
    __classPrivateFieldSet(this, _DiffFile_newFileDiffLines, {}, "f");
    __classPrivateFieldGet(this, _DiffFile_diffLines, "f").forEach((item) => {
        if (item.newLineNumber) {
            this.diffLineLength = Math.max(this.diffLineLength, item.newLineNumber);
            __classPrivateFieldGet(this, _DiffFile_newFileDiffLines, "f")[item.newLineNumber] = item;
        }
    });
}, _DiffFile_composeSyntax = function _DiffFile_composeSyntax({ autoDetectLang, registerHighlighter, }) {
    var _a, _b, _c, _d;
    (_a = __classPrivateFieldGet(this, _DiffFile_oldFileResult, "f")) === null || _a === void 0 ? void 0 : _a.doSyntax({ autoDetectLang, registerHighlighter });
    __classPrivateFieldSet(this, _DiffFile_oldFileSyntaxLines, (_b = __classPrivateFieldGet(this, _DiffFile_oldFileResult, "f")) === null || _b === void 0 ? void 0 : _b.syntaxFile, "f");
    (_c = __classPrivateFieldGet(this, _DiffFile_newFileResult, "f")) === null || _c === void 0 ? void 0 : _c.doSyntax({ autoDetectLang, registerHighlighter });
    __classPrivateFieldSet(this, _DiffFile_newFileSyntaxLines, (_d = __classPrivateFieldGet(this, _DiffFile_newFileResult, "f")) === null || _d === void 0 ? void 0 : _d.syntaxFile, "f");
}, _DiffFile_getOldDiffLine = function _DiffFile_getOldDiffLine(lineNumber) {
    var _a;
    if (!lineNumber)
        return;
    return (_a = __classPrivateFieldGet(this, _DiffFile_oldFileDiffLines, "f")) === null || _a === void 0 ? void 0 : _a[lineNumber];
}, _DiffFile_getNewDiffLine = function _DiffFile_getNewDiffLine(lineNumber) {
    var _a;
    if (!lineNumber)
        return;
    return (_a = __classPrivateFieldGet(this, _DiffFile_newFileDiffLines, "f")) === null || _a === void 0 ? void 0 : _a[lineNumber];
}, _DiffFile_getOldRawLine = function _DiffFile_getOldRawLine(lineNumber) {
    var _a;
    return (_a = __classPrivateFieldGet(this, _DiffFile_oldFileLines, "f")) === null || _a === void 0 ? void 0 : _a[lineNumber];
}, _DiffFile_getNewRawLine = function _DiffFile_getNewRawLine(lineNumber) {
    var _a;
    return (_a = __classPrivateFieldGet(this, _DiffFile_newFileLines, "f")) === null || _a === void 0 ? void 0 : _a[lineNumber];
};

var DiffFileLineType;
(function (DiffFileLineType) {
    DiffFileLineType[DiffFileLineType["hunk"] = 1] = "hunk";
    DiffFileLineType[DiffFileLineType["content"] = 2] = "content";
    DiffFileLineType[DiffFileLineType["widget"] = 3] = "widget";
    DiffFileLineType[DiffFileLineType["extend"] = 4] = "extend";
})(DiffFileLineType || (DiffFileLineType = {}));
const getSplitLines = (diffFile, options) => {
    const splitLineLength = diffFile.splitLineLength;
    const splitLines = [];
    numIterator(splitLineLength, (index) => {
        var _a, _b;
        const hunkLine = diffFile.getSplitHunkLine(index);
        const splitLeftLine = diffFile.getSplitLeftLine(index);
        const splitRightLine = diffFile.getSplitRightLine(index);
        const widgetLine = (options === null || options === void 0 ? void 0 : options.hasRenderWidget) && ((_a = options.widgetData) === null || _a === void 0 ? void 0 : _a[index + 1]);
        const extendLine = (options === null || options === void 0 ? void 0 : options.hasRenderExtend) && ((_b = options.extendData) === null || _b === void 0 ? void 0 : _b[index + 1]);
        hunkLine &&
            hunkLine.splitInfo &&
            hunkLine.splitInfo.startHiddenIndex < hunkLine.splitInfo.endHiddenIndex &&
            splitLines.push({ type: DiffFileLineType.hunk, index, lineNumber: index + 1, hunkLine: hunkLine });
        hunkLine &&
            !hunkLine.splitInfo &&
            !hunkLine.unifiedInfo &&
            hunkLine.type === DiffLineType.Hunk &&
            splitLines.push({ type: DiffFileLineType.hunk, index, lineNumber: index + 1, hunkLine: hunkLine });
        !(splitLeftLine === null || splitLeftLine === void 0 ? void 0 : splitLeftLine.isHidden) &&
            !(splitRightLine === null || splitRightLine === void 0 ? void 0 : splitRightLine.isHidden) &&
            splitLines.push({
                type: DiffFileLineType.content,
                index,
                lineNumber: index + 1,
                splitLine: { left: splitLeftLine, right: splitRightLine },
            });
        widgetLine &&
            splitLines.push({ type: DiffFileLineType.widget, index, lineNumber: index + 1, widgetLine: widgetLine });
        extendLine &&
            splitLines.push({ type: DiffFileLineType.extend, index, lineNumber: index + 1, extendLine: extendLine });
    });
    return splitLines;
};
const getSplitContentLines = (diffFile) => {
    const lines = getSplitLines(diffFile, {});
    return lines.filter((line) => line.type === DiffFileLineType.content);
};
const getUnifiedLines = (diffFile, options) => {
    const unifiedLineLength = diffFile.unifiedLineLength;
    const unifiedLines = [];
    numIterator(unifiedLineLength, (index) => {
        var _a, _b;
        const hunkLine = diffFile.getUnifiedHunkLine(index);
        const unifiedLine = diffFile.getUnifiedLine(index);
        const widgetLine = (options === null || options === void 0 ? void 0 : options.hasRenderWidget) && ((_a = options.widgetData) === null || _a === void 0 ? void 0 : _a[index + 1]);
        const extendLine = (options === null || options === void 0 ? void 0 : options.hasRenderExtend) && ((_b = options.extendData) === null || _b === void 0 ? void 0 : _b[index + 1]);
        hunkLine &&
            hunkLine.unifiedInfo &&
            hunkLine.unifiedInfo.startHiddenIndex < hunkLine.unifiedInfo.endHiddenIndex &&
            unifiedLines.push({ type: DiffFileLineType.hunk, index, lineNumber: index + 1, hunkLine: hunkLine });
        hunkLine &&
            !hunkLine.splitInfo &&
            !hunkLine.unifiedInfo &&
            hunkLine.type === DiffLineType.Hunk &&
            unifiedLines.push({ type: DiffFileLineType.hunk, index, lineNumber: index + 1, hunkLine: hunkLine });
        !unifiedLine.isHidden &&
            unifiedLines.push({ type: DiffFileLineType.content, index, lineNumber: index + 1, unifiedLine: unifiedLine });
        widgetLine &&
            unifiedLines.push({ type: DiffFileLineType.widget, index, lineNumber: index + 1, widgetLine: widgetLine });
        extendLine &&
            unifiedLines.push({ type: DiffFileLineType.extend, index, lineNumber: index + 1, extendLine: extendLine });
    });
    return unifiedLines;
};
const getUnifiedContentLine = (diffFile) => {
    const lines = getUnifiedLines(diffFile, {});
    return lines.filter((line) => line.type === DiffFileLineType.content);
};

const versions = "0.0.7";

const useUnmount = (cb, deps) => {
    const ref = useRef(cb);
    ref.current = cb;
    useEffect(() => ref.current, deps);
};

var DiffModeEnum;
(function (DiffModeEnum) {
    DiffModeEnum[DiffModeEnum["Split"] = 1] = "Split";
    DiffModeEnum[DiffModeEnum["Unified"] = 2] = "Unified";
})(DiffModeEnum || (DiffModeEnum = {}));
const DiffViewContext = createContext(null);
DiffViewContext.displayName = "DiffViewContext";
const useDiffViewContext = () => useContext(DiffViewContext);

const isClient = typeof window !== "undefined";
const useSafeLayout = isClient ? useLayoutEffect : useEffect;

var _TextMeasure_instances, _TextMeasure_key, _TextMeasure_map, _TextMeasure_getInstance;
let canvasCtx = null;
class TextMeasure {
    constructor() {
        _TextMeasure_instances.add(this);
        _TextMeasure_key.set(this, "");
        _TextMeasure_map.set(this, {});
    }
    measure(text, font) {
        const currentKey = `${font === null || font === void 0 ? void 0 : font.fontFamily}-${font === null || font === void 0 ? void 0 : font.fontStyle}-${font === null || font === void 0 ? void 0 : font.fontSize}-${text}`;
        if (__classPrivateFieldGet$1(this, _TextMeasure_map, "f")[currentKey]) {
            return __classPrivateFieldGet$1(this, _TextMeasure_map, "f")[currentKey];
        }
        const instance = __classPrivateFieldGet$1(this, _TextMeasure_instances, "m", _TextMeasure_getInstance).call(this);
        if (font) {
            const currentFontKey = `${font.fontFamily}-${font.fontStyle}-${font.fontSize}`;
            if (__classPrivateFieldGet$1(this, _TextMeasure_key, "f") !== currentFontKey) {
                __classPrivateFieldSet$1(this, _TextMeasure_key, currentFontKey, "f");
                instance.font = `${font.fontStyle || ""} ${font.fontSize || ""} ${font.fontFamily || ""}`;
            }
        }
        else {
            instance.font = "";
        }
        const textWidth = instance.measureText(text).width;
        return textWidth;
    }
}
_TextMeasure_key = new WeakMap(), _TextMeasure_map = new WeakMap(), _TextMeasure_instances = new WeakSet(), _TextMeasure_getInstance = function _TextMeasure_getInstance() {
    canvasCtx = canvasCtx || document.createElement("canvas").getContext("2d");
    return canvasCtx;
};
const measureInstance = new TextMeasure();
const useTextWidth = ({ text, font, }) => {
    const [width, setWidth] = useState(0);
    useSafeLayout(() => {
        const width = measureInstance.measure(text, font);
        setWidth(width);
    }, [text, font]);
    return width;
};

const useDomWidth = ({ selector, enable }) => {
    const [width, setWidth] = useState(0);
    const { useDiffContext } = useDiffViewContext();
    const id = useDiffContext(useCallback((s) => s.id, []));
    useEffect(() => {
        if (enable) {
            const container = document.querySelector(`#diff-root${id}`);
            const wrapper = container === null || container === void 0 ? void 0 : container.querySelector(selector);
            if (!wrapper)
                return;
            const typedWrapper = wrapper;
            const cb = () => {
                var _a;
                const rect = wrapper === null || wrapper === void 0 ? void 0 : wrapper.getBoundingClientRect();
                setWidth((_a = rect === null || rect === void 0 ? void 0 : rect.width) !== null && _a !== void 0 ? _a : 0);
            };
            cb();
            const cleanCb = () => {
                var _a;
                typedWrapper.__observeCallback.delete(cb);
                if (typedWrapper.__observeCallback.size === 0) {
                    (_a = typedWrapper.__observeInstance) === null || _a === void 0 ? void 0 : _a.disconnect();
                    typedWrapper.removeAttribute("data-observe");
                    delete typedWrapper.__observeCallback;
                    delete typedWrapper.__observeInstance;
                }
            };
            if (typedWrapper.__observeCallback) {
                typedWrapper.__observeCallback.add(cb);
                return () => cleanCb();
            }
            typedWrapper.__observeCallback = new Set();
            typedWrapper.__observeCallback.add(cb);
            const observer = new ResizeObserver(() => typedWrapper.__observeCallback.forEach((cb) => cb()));
            typedWrapper.__observeInstance = observer;
            observer.observe(typedWrapper);
            typedWrapper.setAttribute("data-observe", "height");
            return () => cleanCb();
        }
    }, [selector, enable, id]);
    return width;
};

const useSyncHeight = ({ selector, side, enable, }) => {
    const { useDiffContext } = useDiffViewContext();
    const id = useDiffContext(useCallback((s) => s.id, []));
    useSafeLayout(() => {
        if (enable) {
            const container = document.querySelector(`#diff-root${id}`);
            const elements = Array.from((container === null || container === void 0 ? void 0 : container.querySelectorAll(selector)) || []);
            if (elements.length === 2) {
                const ele1 = elements[0];
                const ele2 = elements[1];
                const cb = () => {
                    ele1.style.height = "auto";
                    ele2.style.height = "auto";
                    const rect1 = ele1.getBoundingClientRect();
                    const rect2 = ele2.getBoundingClientRect();
                    if (rect1.height !== rect2.height) {
                        const maxHeight = Math.max(rect1.height, rect2.height);
                        ele1.style.height = maxHeight + "px";
                        ele2.style.height = maxHeight + "px";
                        ele1.setAttribute("data-sync-height", String(maxHeight));
                        ele2.setAttribute("data-sync-height", String(maxHeight));
                    }
                    else {
                        ele1.removeAttribute("data-sync-height");
                        ele2.removeAttribute("data-sync-height");
                    }
                };
                cb();
                const observer = new ResizeObserver(cb);
                const target = ele1.getAttribute("data-side") === side ? ele1 : ele2;
                observer.observe(target);
                target.setAttribute("data-observe", "height");
                return () => {
                    observer.disconnect();
                    target === null || target === void 0 ? void 0 : target.removeAttribute("data-observe");
                };
            }
        }
    }, [selector, enable, side, id]);
};

const addContentBGName = "--diff-add-content--";
const delContentBGName = "--diff-del-content--";
const addLineNumberBGName = "--diff-add-lineNumber--";
const delLineNumberBGName = "--diff-del-lineNumber--";
const plainContentBGName = "--diff-plain-content--";
const expandContentBGName = "--diff-expand-content--";
const plainLineNumberColorName = "--diff-plain-lineNumber-color--";
const plainLineNumberBGName = "--diff-plain-lineNumber--";
const hunkContentBGName = "--diff-hunk-content--";
const hunkContentColorName = "--diff-hunk-content-color--";
const hunkLineNumberBGName = "--diff-hunk-lineNumber--";
const addContentHighlightBGName = "--diff-add-content-highlight--";
const delContentHighlightBGName = "--diff-del-content-highlight--";
const addWidgetBGName = "--diff-add-widget--";
const addWidgetColorName = "--diff-add-widget-color--";
const emptyBGName = "--diff-empty-content--";
const getContentBG = (isAdded, isDelete, hasDiff) => {
    return isAdded
        ? `var(${addContentBGName})`
        : isDelete
            ? `var(${delContentBGName})`
            : hasDiff
                ? `var(${plainContentBGName})`
                : `var(${expandContentBGName})`;
};
const getLineNumberBG = (isAdded, isDelete, hasDiff) => {
    return isAdded
        ? `var(${addLineNumberBGName})`
        : isDelete
            ? `var(${delLineNumberBGName})`
            : hasDiff
                ? `var(${plainLineNumberBGName})`
                : `var(${expandContentBGName})`;
};

const _DiffSplitExtendLine$1 = ({ index, diffFile, side, lineNumber, }) => {
    var _a, _b;
    const { useDiffContext } = useDiffViewContext();
    const { extendData, renderExtendLine } = useDiffContext(React.useCallback((s) => ({ extendData: s.extendData, renderExtendLine: s.renderExtendLine }), []));
    const oldLine = diffFile.getSplitLeftLine(index);
    const newLine = diffFile.getSplitRightLine(index);
    const oldLineExtend = (_a = extendData === null || extendData === void 0 ? void 0 : extendData.oldFile) === null || _a === void 0 ? void 0 : _a[oldLine === null || oldLine === void 0 ? void 0 : oldLine.lineNumber];
    const newLineExtend = (_b = extendData === null || extendData === void 0 ? void 0 : extendData.newFile) === null || _b === void 0 ? void 0 : _b[newLine === null || newLine === void 0 ? void 0 : newLine.lineNumber];
    const currentExtend = side === SplitSide.old ? oldLineExtend : newLineExtend;
    const currentLineNumber = side === SplitSide.old ? oldLine.lineNumber : newLine.lineNumber;
    const otherSide = side === SplitSide.old ? SplitSide.new : SplitSide.old;
    useSyncHeight({
        selector: `tr[data-line="${lineNumber}-extend"]`,
        side: currentExtend ? SplitSide[side] : SplitSide[otherSide],
        enable: side === SplitSide.new && typeof renderExtendLine === "function",
    });
    const width = useDomWidth({
        selector: side === SplitSide.old ? ".old-diff-table-wrapper" : ".new-diff-table-wrapper",
        enable: !!currentExtend && typeof renderExtendLine === "function",
    });
    if (!renderExtendLine)
        return null;
    return (React.createElement("tr", { "data-line": `${lineNumber}-extend`, "data-state": "extend", "data-side": SplitSide[side], className: "diff-line diff-line-extend" }, currentExtend ? (React.createElement("td", { className: `diff-line-extend-${SplitSide[side]}-content p-0`, colSpan: 2 },
        React.createElement("div", { className: "diff-line-extend-wrapper sticky left-0", style: { width } }, width > 0 &&
            (renderExtendLine === null || renderExtendLine === void 0 ? void 0 : renderExtendLine({
                diffFile,
                side,
                lineNumber: currentLineNumber,
                data: currentExtend.data,
                onUpdate: diffFile.notifyAll,
            }))))) : (React.createElement("td", { className: `diff-line-extend-${SplitSide[side]}-placeholder p-0 select-none`, style: { backgroundColor: `var(${emptyBGName})` }, colSpan: 2 },
        React.createElement("span", null, "\u2002")))));
};
const DiffSplitExtendLine$1 = ({ index, diffFile, side, lineNumber, }) => {
    const { useDiffContext } = useDiffViewContext();
    const oldLine = diffFile.getSplitLeftLine(index);
    const newLine = diffFile.getSplitRightLine(index);
    const hasExtend = useDiffContext(React.useCallback((s) => { var _a, _b, _c, _d; return ((_b = (_a = s.extendData) === null || _a === void 0 ? void 0 : _a.oldFile) === null || _b === void 0 ? void 0 : _b[oldLine === null || oldLine === void 0 ? void 0 : oldLine.lineNumber]) || ((_d = (_c = s.extendData) === null || _c === void 0 ? void 0 : _c.newFile) === null || _d === void 0 ? void 0 : _d[newLine === null || newLine === void 0 ? void 0 : newLine.lineNumber]); }, [oldLine === null || oldLine === void 0 ? void 0 : oldLine.lineNumber, newLine === null || newLine === void 0 ? void 0 : newLine.lineNumber]));
    // if the expand action not enabled, the `isHidden` property will never change
    const enableExpand = diffFile.getExpandEnabled();
    const currentLine = side === SplitSide.old ? oldLine : newLine;
    const currentIsShow = hasExtend && (!currentLine.isHidden || !enableExpand);
    if (!currentIsShow)
        return null;
    return React.createElement(_DiffSplitExtendLine$1, { index: index, diffFile: diffFile, side: side, lineNumber: lineNumber });
};

const ExpandDown = ({ className }) => {
    return (React.createElement("svg", { "aria-hidden": "true", height: "16", viewBox: "0 0 16 16", version: "1.1", width: "16", className: className },
        React.createElement("path", { d: "m8.177 14.323 2.896-2.896a.25.25 0 0 0-.177-.427H8.75V7.764a.75.75 0 1 0-1.5 0V11H5.104a.25.25 0 0 0-.177.427l2.896 2.896a.25.25 0 0 0 .354 0ZM2.25 5a.75.75 0 0 0 0-1.5h-.5a.75.75 0 0 0 0 1.5h.5ZM6 4.25a.75.75 0 0 1-.75.75h-.5a.75.75 0 0 1 0-1.5h.5a.75.75 0 0 1 .75.75ZM8.25 5a.75.75 0 0 0 0-1.5h-.5a.75.75 0 0 0 0 1.5h.5ZM12 4.25a.75.75 0 0 1-.75.75h-.5a.75.75 0 0 1 0-1.5h.5a.75.75 0 0 1 .75.75Zm2.25.75a.75.75 0 0 0 0-1.5h-.5a.75.75 0 0 0 0 1.5h.5Z" })));
};
const ExpandUp = ({ className }) => {
    return (React.createElement("svg", { "aria-hidden": "true", height: "16", viewBox: "0 0 16 16", version: "1.1", width: "16", className: className },
        React.createElement("path", { d: "M7.823 1.677 4.927 4.573A.25.25 0 0 0 5.104 5H7.25v3.236a.75.75 0 1 0 1.5 0V5h2.146a.25.25 0 0 0 .177-.427L8.177 1.677a.25.25 0 0 0-.354 0ZM13.75 11a.75.75 0 0 0 0 1.5h.5a.75.75 0 0 0 0-1.5h-.5Zm-3.75.75a.75.75 0 0 1 .75-.75h.5a.75.75 0 0 1 0 1.5h-.5a.75.75 0 0 1-.75-.75ZM7.75 11a.75.75 0 0 0 0 1.5h.5a.75.75 0 0 0 0-1.5h-.5ZM4 11.75a.75.75 0 0 1 .75-.75h.5a.75.75 0 0 1 0 1.5h-.5a.75.75 0 0 1-.75-.75ZM1.75 11a.75.75 0 0 0 0 1.5h.5a.75.75 0 0 0 0-1.5h-.5Z" })));
};
const ExpandAll = ({ className }) => {
    return (React.createElement("svg", { "aria-hidden": "true", height: "16", viewBox: "0 0 16 16", version: "1.1", width: "16", className: className },
        React.createElement("path", { d: "m8.177.677 2.896 2.896a.25.25 0 0 1-.177.427H8.75v1.25a.75.75 0 0 1-1.5 0V4H5.104a.25.25 0 0 1-.177-.427L7.823.677a.25.25 0 0 1 .354 0ZM7.25 10.75a.75.75 0 0 1 1.5 0V12h2.146a.25.25 0 0 1 .177.427l-2.896 2.896a.25.25 0 0 1-.354 0l-2.896-2.896A.25.25 0 0 1 5.104 12H7.25v-1.25Zm-5-2a.75.75 0 0 0 0-1.5h-.5a.75.75 0 0 0 0 1.5h.5ZM6 8a.75.75 0 0 1-.75.75h-.5a.75.75 0 0 1 0-1.5h.5A.75.75 0 0 1 6 8Zm2.25.75a.75.75 0 0 0 0-1.5h-.5a.75.75 0 0 0 0 1.5h.5ZM12 8a.75.75 0 0 1-.75.75h-.5a.75.75 0 0 1 0-1.5h.5A.75.75 0 0 1 12 8Zm2.25.75a.75.75 0 0 0 0-1.5h-.5a.75.75 0 0 0 0 1.5h.5Z" })));
};

const _DiffSplitHunkLine = ({ index, diffFile, side, lineNumber, }) => {
    var _a;
    const currentHunk = diffFile.getSplitHunkLine(index);
    const expandEnabled = diffFile.getExpandEnabled();
    useSyncHeight({
        selector: `tr[data-line="${lineNumber}-hunk"]`,
        side: SplitSide[SplitSide.old],
        enable: side === SplitSide.new,
    });
    const enableHunkAction = side === SplitSide.old;
    const couldExpand = expandEnabled && currentHunk && currentHunk.splitInfo;
    const isExpandAll = currentHunk &&
        currentHunk.splitInfo &&
        currentHunk.splitInfo.endHiddenIndex - currentHunk.splitInfo.startHiddenIndex < composeLen;
    const isFirstLine = currentHunk && currentHunk.index === 0;
    const isLastLine = currentHunk && currentHunk.isLast;
    return (React.createElement("tr", { "data-line": `${lineNumber}-hunk`, "data-state": "hunk", "data-side": SplitSide[side], className: "diff-line diff-line-hunk" }, enableHunkAction ? (React.createElement(React.Fragment, null,
        React.createElement("td", { className: "diff-line-hunk-action sticky left-0 p-[1px] w-[1%] min-w-[40px] select-none", style: {
                backgroundColor: `var(${hunkLineNumberBGName})`,
                color: `var(${plainLineNumberColorName})`,
            } }, couldExpand ? (isFirstLine ? (React.createElement("button", { className: "w-full diff-widget-tooltip hover:bg-blue-300 flex justify-center items-center py-[6px] cursor-pointer rounded-[2px]", title: "Expand Up", "data-title": "Expand Up", onClick: () => diffFile.onSplitHunkExpand("up", index) },
            React.createElement(ExpandUp, { className: "fill-current" }))) : isLastLine ? (React.createElement("button", { className: "w-full diff-widget-tooltip hover:bg-blue-300 flex justify-center items-center py-[6px] cursor-pointer rounded-[2px] relative", title: "Expand Down", "data-title": "Expand Down", onClick: () => {
                diffFile.onSplitHunkExpand("down", index);
            } },
            React.createElement(ExpandDown, { className: "fill-current" }))) : isExpandAll ? (React.createElement("button", { className: "w-full diff-widget-tooltip hover:bg-blue-300 flex justify-center items-center py-[6px] cursor-pointer rounded-[2px]", title: "Expand All", "data-title": "Expand All", onClick: () => diffFile.onSplitHunkExpand("all", index) },
            React.createElement(ExpandAll, { className: "fill-current" }))) : (React.createElement(React.Fragment, null,
            React.createElement("button", { className: "w-full diff-widget-tooltip hover:bg-blue-300 flex justify-center items-center py-[2px] cursor-pointer rounded-[2px]", title: "Expand Down", "data-title": "Expand Down", onClick: () => diffFile.onSplitHunkExpand("down", index) },
                React.createElement(ExpandDown, { className: "fill-current" })),
            React.createElement("button", { className: "w-full diff-widget-tooltip hover:bg-blue-300 flex justify-center items-center py-[2px] cursor-pointer rounded-[2px]", title: "Expand Up", "data-title": "Expand Up", onClick: () => diffFile.onSplitHunkExpand("up", index) },
                React.createElement(ExpandUp, { className: "fill-current" }))))) : (React.createElement("div", { className: "min-h-[28px]" }, "\u2002"))),
        React.createElement("td", { className: "diff-line-hunk-content pr-[10px] align-middle", style: { backgroundColor: `var(${hunkContentBGName})` } },
            React.createElement("div", { className: "pl-[1.5em]", style: {
                    color: `var(${hunkContentColorName})`,
                } }, ((_a = currentHunk.splitInfo) === null || _a === void 0 ? void 0 : _a.plainText) || currentHunk.text)))) : (React.createElement("td", { className: "diff-line-hunk-placeholder select-none", colSpan: 2, style: { backgroundColor: `var(${hunkContentBGName})` } },
        React.createElement("div", { className: "min-h-[28px]" }, "\u2002")))));
};
const DiffSplitHunkLine$1 = ({ index, diffFile, side, lineNumber, }) => {
    const currentHunk = diffFile.getSplitHunkLine(index);
    const currentIsShow = currentHunk &&
        currentHunk.splitInfo &&
        currentHunk.splitInfo.startHiddenIndex < currentHunk.splitInfo.endHiddenIndex;
    const currentIsPureHunk = currentHunk && diffFile._getIsPureDiffRender() && !currentHunk.splitInfo;
    if (!currentIsShow && !currentIsPureHunk)
        return null;
    return React.createElement(_DiffSplitHunkLine, { index: index, diffFile: diffFile, side: side, lineNumber: lineNumber });
};

const DiffSplitAddWidget = ({ side, className, lineNumber, onWidgetClick, onOpenAddWidget, }) => {
    return (React.createElement("div", { className: "diff-add-widget-wrapper" + (className ? " " + className : ""), style: {
            width: "calc(var(--diff-font-size--) * 1.4)",
            height: "calc(var(--diff-font-size--) * 1.4)",
        } },
        React.createElement("button", { className: "diff-add-widget w-0 h-0 overflow-hidden cursor-pointer rounded-md flex items-center justify-center transition-transform origin-center group-hover:w-full group-hover:h-full hover:scale-110", style: {
                color: `var(${addWidgetColorName})`,
                zIndex: 1,
                fontSize: `1.2em`,
                backgroundColor: `var(${addWidgetBGName})`,
            }, onClick: () => {
                onOpenAddWidget(lineNumber, side);
                onWidgetClick === null || onWidgetClick === void 0 ? void 0 : onWidgetClick(lineNumber, side);
            } }, "+")));
};
const DiffUnifiedAddWidget = ({ lineNumber, side, onWidgetClick, onOpenAddWidget, }) => {
    return (React.createElement("div", { className: "diff-add-widget-wrapper absolute left-[100%] top-[1px] translate-x-[-50%]", style: {
            width: "calc(var(--diff-font-size--) * 1.4)",
            height: "calc(var(--diff-font-size--) * 1.4)",
        } },
        React.createElement("button", { className: "diff-add-widget overflow-hidden cursor-pointer rounded-md w-0 h-0 flex items-center justify-center transition-transform origin-center group-hover:w-full group-hover:h-full hover:scale-110", style: {
                color: `var(${addWidgetColorName})`,
                zIndex: 1,
                fontSize: `1.2em`,
                backgroundColor: `var(${addWidgetBGName})`,
            }, onClick: () => {
                onOpenAddWidget(lineNumber, side);
                onWidgetClick === null || onWidgetClick === void 0 ? void 0 : onWidgetClick(lineNumber, side);
            } }, "+")));
};

const DiffString = ({ rawLine, diffLine, operator, }) => {
    const range = diffLine === null || diffLine === void 0 ? void 0 : diffLine.range;
    if (range) {
        const str1 = rawLine.slice(0, range.location);
        const str2 = rawLine.slice(range.location, range.location + range.length);
        const str3 = rawLine.slice(range.location + range.length);
        const isNewLineSymbolChanged = range.isNewLineSymbolChanged;
        return (React.createElement("span", { className: "diff-line-content-raw" },
            React.createElement("span", { "data-range-start": range.location, "data-range-end": range.location + range.length },
                str1,
                React.createElement("span", { "data-diff-highlight": true, className: "rounded-[0.2em]", style: {
                        backgroundColor: operator === "add" ? `var(${addContentHighlightBGName})` : `var(${delContentHighlightBGName})`,
                    } }, isNewLineSymbolChanged
                    ? str2 === "\r"
                        ? "␍"
                        : str2 === "\n"
                            ? "␊"
                            : str2 === "\r\n"
                                ? "␍␊"
                                : str2
                    : str2),
                str3)));
    }
    return React.createElement("span", { className: "diff-line-content-raw" }, rawLine);
};
const DiffSyntax = ({ rawLine, diffLine, operator, syntaxLine, }) => {
    var _a, _b;
    if (!syntaxLine) {
        return React.createElement(DiffString, { rawLine: rawLine, diffLine: diffLine, operator: operator });
    }
    const range = diffLine === null || diffLine === void 0 ? void 0 : diffLine.range;
    if (range) {
        return (React.createElement("span", { className: "diff-line-syntax-raw" },
            React.createElement("span", { "data-range-start": range.location, "data-range-end": range.location + range.length }, (_a = syntaxLine.nodeList) === null || _a === void 0 ? void 0 : _a.map(({ node, wrapper }, index) => {
                var _a, _b, _c, _d;
                if (node.endIndex < range.location || range.location + range.length < node.startIndex) {
                    return (React.createElement("span", { key: index, "data-start": node.startIndex, "data-end": node.endIndex, className: (_b = (_a = wrapper === null || wrapper === void 0 ? void 0 : wrapper.properties) === null || _a === void 0 ? void 0 : _a.className) === null || _b === void 0 ? void 0 : _b.join(" ") }, node.value));
                }
                else {
                    const index1 = range.location - node.startIndex;
                    const index2 = index1 < 0 ? 0 : index1;
                    const str1 = node.value.slice(0, index2);
                    const str2 = node.value.slice(index2, index1 + range.length);
                    const str3 = node.value.slice(index1 + range.length);
                    const isStart = str1.length || range.location === node.startIndex;
                    const isEnd = str3.length || node.endIndex === range.location + range.length - 1;
                    const isNewLineSymbolChanged = range.isNewLineSymbolChanged;
                    return (React.createElement("span", { key: index, "data-start": node.startIndex, "data-end": node.endIndex, className: (_d = (_c = wrapper === null || wrapper === void 0 ? void 0 : wrapper.properties) === null || _c === void 0 ? void 0 : _c.className) === null || _d === void 0 ? void 0 : _d.join(" ") },
                        str1,
                        React.createElement("span", { "data-diff-highlight": true, style: {
                                backgroundColor: operator === "add" ? `var(${addContentHighlightBGName})` : `var(${delContentHighlightBGName})`,
                                borderTopLeftRadius: isStart ? "0.2em" : undefined,
                                borderBottomLeftRadius: isStart ? "0.2em" : undefined,
                                borderTopRightRadius: isEnd ? "0.2em" : undefined,
                                borderBottomRightRadius: isEnd ? "0.2em" : undefined,
                            } }, isNewLineSymbolChanged
                            ? str2 === "\r"
                                ? "␍"
                                : str2 === "\n"
                                    ? "␊"
                                    : str2 === "\r\n"
                                        ? "␍␊"
                                        : str2
                            : str2),
                        str3));
                }
            }))));
    }
    return (React.createElement("span", { className: "diff-line-syntax-raw" }, (_b = syntaxLine === null || syntaxLine === void 0 ? void 0 : syntaxLine.nodeList) === null || _b === void 0 ? void 0 : _b.map(({ node, wrapper }, index) => {
        var _a, _b;
        return (React.createElement("span", { key: index, "data-start": node.startIndex, "data-end": node.endIndex, className: (_b = (_a = wrapper === null || wrapper === void 0 ? void 0 : wrapper.properties) === null || _a === void 0 ? void 0 : _a.className) === null || _b === void 0 ? void 0 : _b.join(" ") }, node.value));
    })));
};
const DiffContent = ({ diffLine, rawLine, syntaxLine, enableWrap, enableHighlight, }) => {
    var _a;
    const isAdded = (diffLine === null || diffLine === void 0 ? void 0 : diffLine.type) === DiffLineType.Add;
    const isDelete = (diffLine === null || diffLine === void 0 ? void 0 : diffLine.type) === DiffLineType.Delete;
    const isMaxLineLengthToIgnoreSyntax = ((_a = syntaxLine === null || syntaxLine === void 0 ? void 0 : syntaxLine.nodeList) === null || _a === void 0 ? void 0 : _a.length) > 150;
    return (React.createElement("div", { className: "diff-line-content-item pl-[2.0em]", "data-val": rawLine, style: {
            whiteSpace: enableWrap ? "pre-wrap" : "pre",
            wordBreak: enableWrap ? "break-all" : "initial",
        } },
        React.createElement("span", { "data-operator": isAdded ? "+" : isDelete ? "-" : undefined, className: "diff-line-content-operator inline-block w-[1.5em] ml-[-1.5em] indent-[0.2em] select-none" }, isAdded ? "+" : isDelete ? "-" : " "),
        enableHighlight && syntaxLine && !isMaxLineLengthToIgnoreSyntax ? (React.createElement(DiffSyntax, { operator: isAdded ? "add" : isDelete ? "del" : undefined, rawLine: rawLine, diffLine: diffLine, syntaxLine: syntaxLine })) : (React.createElement(DiffString, { operator: isAdded ? "add" : isDelete ? "del" : undefined, rawLine: rawLine, diffLine: diffLine }))));
};

const DiffWidgetContext = createContext(null);
DiffWidgetContext.displayName = "DiffWidgetContext";
const useDiffWidgetContext = () => useContext(DiffWidgetContext);

const _DiffSplitLine$1 = ({ index, diffFile, lineNumber, side, }) => {
    var _a, _b;
    const getCurrentSyntaxLine = side === SplitSide.old ? diffFile.getOldSyntaxLine : diffFile.getNewSyntaxLine;
    const oldLine = diffFile.getSplitLeftLine(index);
    const newLine = diffFile.getSplitRightLine(index);
    const currentLine = side === SplitSide.old ? oldLine : newLine;
    const hasDiff = !!(currentLine === null || currentLine === void 0 ? void 0 : currentLine.diff);
    const hasContent = !!currentLine.lineNumber;
    const hasChange = checkDiffLineIncludeChange(currentLine === null || currentLine === void 0 ? void 0 : currentLine.diff);
    const isAdded = ((_a = currentLine === null || currentLine === void 0 ? void 0 : currentLine.diff) === null || _a === void 0 ? void 0 : _a.type) === DiffLineType.Add;
    const isDelete = ((_b = currentLine === null || currentLine === void 0 ? void 0 : currentLine.diff) === null || _b === void 0 ? void 0 : _b.type) === DiffLineType.Delete;
    const { useDiffContext } = useDiffViewContext();
    const { enableHighlight, enableAddWidget, onAddWidgetClick } = useDiffContext(React.useCallback((s) => ({
        enableHighlight: s.enableHighlight,
        enableAddWidget: s.enableAddWidget,
        onAddWidgetClick: s.onAddWidgetClick,
    }), []));
    const { useWidget } = useDiffWidgetContext();
    const { setWidget } = useWidget.getReadonlyState();
    const contentBG = getContentBG(isAdded, isDelete, hasDiff);
    const lineNumberBG = getLineNumberBG(isAdded, isDelete, hasDiff);
    const syntaxLine = getCurrentSyntaxLine(currentLine.lineNumber);
    return (React.createElement("tr", { "data-line": lineNumber, "data-state": hasDiff || !hasContent ? "diff" : "plain", "data-side": SplitSide[side], className: "diff-line" + (hasContent ? " group" : "") }, hasContent ? (React.createElement(React.Fragment, null,
        React.createElement("td", { className: `diff-line-${SplitSide[side]}-num sticky left-0 pl-[10px] pr-[10px] text-right align-top select-none w-[1%] min-w-[40px]`, style: {
                backgroundColor: lineNumberBG,
                color: `var(${plainLineNumberColorName})`,
            } },
            hasDiff && enableAddWidget && (React.createElement(DiffSplitAddWidget, { index: index, lineNumber: currentLine.lineNumber, side: side, diffFile: diffFile, onWidgetClick: onAddWidgetClick, className: "absolute left-[100%] translate-x-[-50%] z-[1]", onOpenAddWidget: (lineNumber, side) => setWidget({ lineNumber: lineNumber, side: side }) })),
            React.createElement("span", { "data-line-num": currentLine.lineNumber, style: { opacity: hasChange ? undefined : 0.5 } }, currentLine.lineNumber)),
        React.createElement("td", { className: `diff-line-${SplitSide[side]}-content pr-[10px] align-top`, style: { backgroundColor: contentBG } },
            React.createElement(DiffContent, { enableWrap: false, diffFile: diffFile, rawLine: currentLine.value, diffLine: currentLine.diff, syntaxLine: syntaxLine, enableHighlight: enableHighlight })))) : (React.createElement("td", { className: `diff-line-${SplitSide[side]}-placeholder select-none`, style: { backgroundColor: `var(${emptyBGName})` }, colSpan: 2 },
        React.createElement("span", null, "\u2002")))));
};
const DiffSplitLine$1 = ({ index, diffFile, lineNumber, side, }) => {
    const getCurrentLine = side === SplitSide.old ? diffFile.getSplitLeftLine : diffFile.getSplitRightLine;
    const currentLine = getCurrentLine(index);
    if (currentLine === null || currentLine === void 0 ? void 0 : currentLine.isHidden)
        return null;
    return React.createElement(_DiffSplitLine$1, { index: index, diffFile: diffFile, lineNumber: lineNumber, side: side });
};

const _DiffSplitWidgetLine$1 = ({ diffFile, side, lineNumber, currentLine, setWidget, currentWidget, }) => {
    const otherSide = side === SplitSide.old ? SplitSide.new : SplitSide.old;
    const { useDiffContext } = useDiffViewContext();
    const renderWidgetLine = useDiffContext(React.useCallback((s) => s.renderWidgetLine, []));
    useSyncHeight({
        selector: `tr[data-line="${lineNumber}-widget"]`,
        side: currentWidget ? SplitSide[side] : SplitSide[otherSide],
        enable: side === SplitSide.new && typeof renderWidgetLine === "function",
    });
    const width = useDomWidth({
        selector: side === SplitSide.old ? ".old-diff-table-wrapper" : ".new-diff-table-wrapper",
        enable: !!currentWidget && typeof renderWidgetLine === "function",
    });
    if (!renderWidgetLine)
        return null;
    return (React.createElement("tr", { "data-line": `${lineNumber}-widget`, "data-state": "widget", "data-side": SplitSide[side], className: "diff-line diff-line-widget" }, currentWidget ? (React.createElement("td", { className: `diff-line-widget-${SplitSide[side]}-content p-0`, colSpan: 2 },
        React.createElement("div", { className: "diff-line-widget-wrapper sticky left-0", style: { width } }, width > 0 &&
            (renderWidgetLine === null || renderWidgetLine === void 0 ? void 0 : renderWidgetLine({
                diffFile,
                side,
                lineNumber: currentLine.lineNumber,
                onClose: () => setWidget({}),
            }))))) : (React.createElement("td", { className: `diff-line-widget-${SplitSide[side]}-placeholder p-0`, style: { backgroundColor: `var(${emptyBGName})` }, colSpan: 2 },
        React.createElement("span", null, "\u2002")))));
};
const DiffSplitWidgetLine$1 = ({ index, diffFile, side, lineNumber, }) => {
    const { useWidget } = useDiffWidgetContext();
    const { widgetLineNumber, widgetSide, setWidget } = useWidget(React.useCallback((s) => ({ widgetLineNumber: s.widgetLineNumber, widgetSide: s.widgetSide, setWidget: s.setWidget }), []));
    const oldLine = diffFile.getSplitLeftLine(index);
    const newLine = diffFile.getSplitRightLine(index);
    const oldLineWidget = oldLine.lineNumber && widgetSide === SplitSide.old && widgetLineNumber === oldLine.lineNumber;
    const newLineWidget = newLine.lineNumber && widgetSide === SplitSide.new && widgetLineNumber === newLine.lineNumber;
    const currentLine = side === SplitSide.old ? oldLine : newLine;
    const currentWidget = side === SplitSide.old ? oldLineWidget : newLineWidget;
    const currentIsShow = oldLineWidget || newLineWidget;
    if (!currentIsShow)
        return null;
    return (React.createElement(_DiffSplitWidgetLine$1, { index: index, diffFile: diffFile, side: side, lineNumber: lineNumber, currentLine: currentLine, setWidget: setWidget, currentWidget: currentWidget }));
};

const removeAllSelection = () => {
    const selection = window.getSelection();
    for (let i = 0; i < selection.rangeCount; i++) {
        selection.removeRange(selection.getRangeAt(i));
    }
};
const syncScroll = (left, right) => {
    const onScroll = function (event) {
        if (event === null || event.target === null)
            return;
        if (event.target === left) {
            right.scrollTop = left.scrollTop;
            right.scrollLeft = left.scrollLeft;
        }
        else {
            left.scrollTop = right.scrollTop;
            left.scrollLeft = right.scrollLeft;
        }
    };
    if (!left.onscroll) {
        left.onscroll = onScroll;
    }
    if (!right.onscroll) {
        right.onscroll = onScroll;
    }
    return () => {
        left.onscroll = null;
        right.onscroll = null;
    };
};

const onMouseDown$1 = (e) => {
    const ele = e.target;
    // need remove all the selection
    if (ele && ele instanceof HTMLElement && ele.nodeName === "BUTTON") {
        removeAllSelection();
        return;
    }
};
const DiffSplitViewTable = ({ side, diffFile, width }) => {
    const className = side === SplitSide.new ? "new-diff-table" : "old-diff-table";
    const lines = getSplitContentLines(diffFile);
    return (React.createElement("table", { className: className + " border-collapse w-full", "data-mode": SplitSide[side] },
        React.createElement("colgroup", null,
            React.createElement("col", { className: `diff-table-${SplitSide[side]}-num-col`, style: { minWidth: Math.round(width) + 25 } }),
            React.createElement("col", { className: `diff-table-${SplitSide[side]}-content-col` })),
        React.createElement("thead", { className: "hidden" },
            React.createElement("tr", null,
                React.createElement("th", { scope: "col" },
                    SplitSide[side],
                    " line number"),
                React.createElement("th", { scope: "col" },
                    SplitSide[side],
                    " line content"))),
        React.createElement("tbody", { className: "diff-table-body leading-[1.4]", onMouseDownCapture: onMouseDown$1 },
            lines.map((line) => (React.createElement(Fragment, { key: line.index },
                React.createElement(DiffSplitHunkLine$1, { index: line.index, side: side, lineNumber: line.lineNumber, diffFile: diffFile }),
                React.createElement(DiffSplitLine$1, { index: line.index, side: side, lineNumber: line.lineNumber, diffFile: diffFile }),
                React.createElement(DiffSplitWidgetLine$1, { index: line.index, side: side, lineNumber: line.lineNumber, diffFile: diffFile }),
                React.createElement(DiffSplitExtendLine$1, { index: line.index, side: side, lineNumber: line.lineNumber, diffFile: diffFile })))),
            React.createElement(DiffSplitHunkLine$1, { side: side, index: diffFile.splitLineLength, lineNumber: diffFile.splitLineLength, diffFile: diffFile }))));
};
const DiffSplitViewNormal = memo(({ diffFile }) => {
    const ref1 = useRef(null);
    const ref2 = useRef(null);
    const splitLineLength = diffFile.splitLineLength;
    const { useDiffContext } = useDiffViewContext();
    const fontSize = useDiffContext(useCallback((s) => s.fontSize, []));
    useSyncExternalStore(diffFile.subscribe, diffFile.getUpdateCount);
    useEffect(() => {
        const left = ref1.current;
        const right = ref2.current;
        if (!left || !right)
            return;
        return syncScroll(left, right);
    }, []);
    const width = useTextWidth({
        text: splitLineLength.toString(),
        font: { fontSize: fontSize + "px", fontFamily: "Menlo, Consolas, monospace" },
    });
    return (React.createElement("div", { className: "split-diff-view split-diff-view-wrap w-full flex basis-[50%]" },
        React.createElement("div", { className: "old-diff-table-wrapper overflow-auto w-full scrollbar-hide scrollbar-disable", ref: ref1, style: {
                overscrollBehaviorX: "none",
                fontFamily: "Menlo, Consolas, monospace",
                fontSize: "var(--diff-font-size--)",
            } },
            React.createElement(DiffSplitViewTable, { side: SplitSide.old, diffFile: diffFile, width: width })),
        React.createElement("div", { className: "diff-split-line w-[1.5px] bg-[#ccc]" }),
        React.createElement("div", { className: "new-diff-table-wrapper overflow-auto w-full scrollbar-hide scrollbar-disable", ref: ref2, style: {
                overscrollBehaviorX: "none",
                fontFamily: "Menlo, Consolas, monospace",
                fontSize: "var(--diff-font-size--)",
            } },
            React.createElement(DiffSplitViewTable, { side: SplitSide.new, diffFile: diffFile, width: width }))));
});
DiffSplitViewNormal.displayName = "DiffSplitViewNormal";

const _DiffSplitExtendLine = ({ index, diffFile, lineNumber, }) => {
    var _a, _b;
    const { useDiffContext } = useDiffViewContext();
    // 需要显示的时候才进行方法订阅，可以大幅度提高性能
    const { extendData, renderExtendLine } = useDiffContext(React.useCallback((s) => ({ extendData: s.extendData, renderExtendLine: s.renderExtendLine }), []));
    const oldLine = diffFile.getSplitLeftLine(index);
    const newLine = diffFile.getSplitRightLine(index);
    const oldLineExtend = (_a = extendData === null || extendData === void 0 ? void 0 : extendData.oldFile) === null || _a === void 0 ? void 0 : _a[oldLine === null || oldLine === void 0 ? void 0 : oldLine.lineNumber];
    const newLineExtend = (_b = extendData === null || extendData === void 0 ? void 0 : extendData.newFile) === null || _b === void 0 ? void 0 : _b[newLine === null || newLine === void 0 ? void 0 : newLine.lineNumber];
    if (!renderExtendLine)
        return null;
    return (React.createElement("tr", { "data-line": `${lineNumber}-extend`, "data-state": "extend", className: "diff-line diff-line-extend" },
        oldLineExtend ? (React.createElement("td", { className: "diff-line-extend-old-content p-0", colSpan: 2 },
            React.createElement("div", { className: "diff-line-extend-wrapper" }, renderExtendLine === null || renderExtendLine === void 0 ? void 0 : renderExtendLine({
                diffFile,
                side: SplitSide.old,
                lineNumber: oldLine.lineNumber,
                data: oldLineExtend.data,
                onUpdate: diffFile.notifyAll,
            })))) : (React.createElement("td", { className: "diff-line-extend-old-placeholder p-0 select-none", style: { backgroundColor: `var(${emptyBGName})` }, colSpan: 2 },
            React.createElement("span", null, "\u2002"))),
        newLineExtend ? (React.createElement("td", { className: "diff-line-extend-new-content p-0 border-l-[1px] border-l-[#ccc]", colSpan: 2 },
            React.createElement("div", { className: "diff-line-extend-wrapper" }, renderExtendLine === null || renderExtendLine === void 0 ? void 0 : renderExtendLine({
                diffFile,
                side: SplitSide.new,
                lineNumber: newLine.lineNumber,
                data: newLineExtend.data,
                onUpdate: diffFile.notifyAll,
            })))) : (React.createElement("td", { className: "diff-line-extend-new-placeholder p-0 border-l-[1px] border-l-[#ccc] select-none", style: { backgroundColor: `var(${emptyBGName})` }, colSpan: 2 },
            React.createElement("span", null, "\u2002")))));
};
const DiffSplitExtendLine = ({ index, diffFile, lineNumber, }) => {
    const { useDiffContext } = useDiffViewContext();
    const oldLine = diffFile.getSplitLeftLine(index);
    const newLine = diffFile.getSplitRightLine(index);
    const hasExtend = useDiffContext(React.useCallback((s) => { var _a, _b, _c, _d; return ((_b = (_a = s.extendData) === null || _a === void 0 ? void 0 : _a.oldFile) === null || _b === void 0 ? void 0 : _b[oldLine === null || oldLine === void 0 ? void 0 : oldLine.lineNumber]) || ((_d = (_c = s.extendData) === null || _c === void 0 ? void 0 : _c.newFile) === null || _d === void 0 ? void 0 : _d[newLine === null || newLine === void 0 ? void 0 : newLine.lineNumber]); }, [oldLine === null || oldLine === void 0 ? void 0 : oldLine.lineNumber, newLine === null || newLine === void 0 ? void 0 : newLine.lineNumber]));
    // if the expand action not enabled, the `isHidden` property will never change
    const enableExpand = diffFile.getExpandEnabled();
    const currentIsShow = hasExtend && ((!(oldLine === null || oldLine === void 0 ? void 0 : oldLine.isHidden) && !(newLine === null || newLine === void 0 ? void 0 : newLine.isHidden)) || !enableExpand);
    if (!currentIsShow)
        return null;
    return React.createElement(_DiffSplitExtendLine, { index: index, diffFile: diffFile, lineNumber: lineNumber });
};

const DiffSplitHunkLine = ({ index, diffFile, lineNumber, }) => {
    var _a;
    const currentHunk = diffFile.getSplitHunkLine(index);
    const expandEnabled = diffFile.getExpandEnabled();
    const couldExpand = expandEnabled && currentHunk && currentHunk.splitInfo;
    const isExpandAll = currentHunk &&
        currentHunk.splitInfo &&
        currentHunk.splitInfo.endHiddenIndex - currentHunk.splitInfo.startHiddenIndex < composeLen;
    const currentIsShow = currentHunk &&
        currentHunk.splitInfo &&
        currentHunk.splitInfo.startHiddenIndex < currentHunk.splitInfo.endHiddenIndex;
    const currentIsPureHunk = currentHunk && diffFile._getIsPureDiffRender() && !currentHunk.splitInfo;
    const isFirstLine = currentHunk && currentHunk.index === 0;
    const isLastLine = currentHunk && currentHunk.isLast;
    if (!currentIsShow && !currentIsPureHunk)
        return null;
    return (React.createElement("tr", { "data-line": `${lineNumber}-hunk`, "data-state": "hunk", className: "diff-line diff-line-hunk" },
        React.createElement("td", { className: "diff-line-hunk-action p-[1px] w-[1%] min-w-[40px] select-none", style: {
                backgroundColor: `var(${hunkLineNumberBGName})`,
                color: `var(${plainLineNumberColorName})`,
            } }, couldExpand ? (isFirstLine ? (React.createElement("button", { className: "w-full diff-widget-tooltip hover:bg-blue-300 flex justify-center items-center py-[6px] cursor-pointer rounded-[2px]", title: "Expand Up", "data-title": "Expand Up", onClick: () => diffFile.onSplitHunkExpand("up", index) },
            React.createElement(ExpandUp, { className: "fill-current" }))) : isLastLine ? (React.createElement("button", { className: "w-full diff-widget-tooltip hover:bg-blue-300 flex justify-center items-center py-[6px] cursor-pointer rounded-[2px]", title: "Expand Down", "data-title": "Expand Down", onClick: () => diffFile.onSplitHunkExpand("down", index) },
            React.createElement(ExpandDown, { className: "fill-current" }))) : isExpandAll ? (React.createElement("button", { className: "w-full diff-widget-tooltip hover:bg-blue-300 flex justify-center items-center py-[6px] cursor-pointer rounded-[2px]", title: "Expand All", "data-title": "Expand All", onClick: () => diffFile.onSplitHunkExpand("all", index) },
            React.createElement(ExpandAll, { className: "fill-current" }))) : (React.createElement(React.Fragment, null,
            React.createElement("button", { className: "w-full diff-widget-tooltip hover:bg-blue-300 flex justify-center items-center py-[2px] cursor-pointer rounded-[2px]", title: "Expand Down", "data-title": "Expand Down", onClick: () => diffFile.onSplitHunkExpand("down", index) },
                React.createElement(ExpandDown, { className: "fill-current" })),
            React.createElement("button", { className: "w-full diff-widget-tooltip hover:bg-blue-300 flex justify-center items-center py-[2px] cursor-pointer rounded-[2px]", title: "Expand Up", "data-title": "Expand Up", onClick: () => diffFile.onSplitHunkExpand("up", index) },
                React.createElement(ExpandUp, { className: "fill-current" }))))) : (React.createElement("div", { className: "min-h-[28px]" }, "\u2002"))),
        React.createElement("td", { className: "diff-line-hunk-content pr-[10px] align-middle", style: { backgroundColor: `var(${hunkContentBGName})` }, colSpan: 3 },
            React.createElement("div", { className: "pl-[1.5em]", style: {
                    color: `var(${hunkContentColorName})`,
                } }, ((_a = currentHunk.splitInfo) === null || _a === void 0 ? void 0 : _a.plainText) || currentHunk.text))));
};

const _DiffSplitLine = ({ index, diffFile, lineNumber }) => {
    var _a, _b;
    const oldLine = diffFile.getSplitLeftLine(index);
    const newLine = diffFile.getSplitRightLine(index);
    const oldSyntaxLine = diffFile.getOldSyntaxLine(oldLine === null || oldLine === void 0 ? void 0 : oldLine.lineNumber);
    const newSyntaxLine = diffFile.getNewSyntaxLine(newLine === null || newLine === void 0 ? void 0 : newLine.lineNumber);
    const hasDiff = !!(oldLine === null || oldLine === void 0 ? void 0 : oldLine.diff) || !!(newLine === null || newLine === void 0 ? void 0 : newLine.diff);
    const hasChange = checkDiffLineIncludeChange(oldLine === null || oldLine === void 0 ? void 0 : oldLine.diff) || checkDiffLineIncludeChange(newLine === null || newLine === void 0 ? void 0 : newLine.diff);
    const oldLineIsDelete = ((_a = oldLine === null || oldLine === void 0 ? void 0 : oldLine.diff) === null || _a === void 0 ? void 0 : _a.type) === DiffLineType.Delete;
    const newLineIsAdded = ((_b = newLine === null || newLine === void 0 ? void 0 : newLine.diff) === null || _b === void 0 ? void 0 : _b.type) === DiffLineType.Add;
    const { useDiffContext } = useDiffViewContext();
    const { enableHighlight, enableAddWidget, onAddWidgetClick } = useDiffContext(React.useCallback((s) => ({
        enableHighlight: s.enableHighlight,
        enableAddWidget: s.enableAddWidget,
        onAddWidgetClick: s.onAddWidgetClick,
    }), []));
    const { useWidget } = useDiffWidgetContext();
    const { setWidget } = useWidget.getReadonlyState();
    const hasOldLine = !!oldLine.lineNumber;
    const hasNewLine = !!newLine.lineNumber;
    const oldLineContentBG = getContentBG(false, oldLineIsDelete, hasDiff);
    const oldLineNumberBG = getLineNumberBG(false, oldLineIsDelete, hasDiff);
    const newLineContentBG = getContentBG(newLineIsAdded, false, hasDiff);
    const newLineNumberBG = getLineNumberBG(newLineIsAdded, false, hasDiff);
    return (React.createElement("tr", { "data-line": lineNumber, "data-state": hasDiff ? "diff" : "plain", className: "diff-line" },
        hasOldLine ? (React.createElement(React.Fragment, null,
            React.createElement("td", { className: "diff-line-old-num group relative pl-[10px] pr-[10px] text-right align-top select-none w-[1%] min-w-[40px]", "data-side": SplitSide[SplitSide.old], style: { backgroundColor: oldLineNumberBG, color: `var(${plainLineNumberColorName})` } },
                hasDiff && enableAddWidget && (React.createElement(DiffSplitAddWidget, { index: index, lineNumber: oldLine.lineNumber, side: SplitSide.old, diffFile: diffFile, onWidgetClick: onAddWidgetClick, className: "absolute left-[100%] translate-x-[-50%] z-[1]", onOpenAddWidget: (lineNumber, side) => setWidget({ lineNumber: lineNumber, side: side }) })),
                React.createElement("span", { "data-line-num": oldLine.lineNumber, style: { opacity: hasChange ? undefined : 0.5 } }, oldLine.lineNumber)),
            React.createElement("td", { className: "diff-line-old-content group relative pr-[10px] align-top", "data-side": SplitSide[SplitSide.old], style: { backgroundColor: oldLineContentBG } },
                hasDiff && enableAddWidget && (React.createElement(DiffSplitAddWidget, { index: index, lineNumber: oldLine.lineNumber, side: SplitSide.old, diffFile: diffFile, onWidgetClick: onAddWidgetClick, className: "absolute right-[100%] translate-x-[50%] z-[1] select-none", onOpenAddWidget: (lineNumber, side) => setWidget({ lineNumber: lineNumber, side: side }) })),
                React.createElement(DiffContent, { enableWrap: true, diffFile: diffFile, rawLine: oldLine.value, diffLine: oldLine.diff, syntaxLine: oldSyntaxLine, enableHighlight: enableHighlight })))) : (React.createElement("td", { className: "diff-line-old-placeholder select-none", "data-side": SplitSide[SplitSide.old], style: { backgroundColor: `var(${emptyBGName})` }, colSpan: 2 },
            React.createElement("span", null, "\u2002"))),
        hasNewLine ? (React.createElement(React.Fragment, null,
            React.createElement("td", { className: "diff-line-new-num group relative pl-[10px] pr-[10px] text-right align-top select-none w-[1%] min-w-[40px] border-l-[1px] border-l-[#ccc]", "data-side": SplitSide[SplitSide.new], style: { backgroundColor: newLineNumberBG, color: `var(${plainLineNumberColorName})` } },
                hasDiff && enableAddWidget && (React.createElement(DiffSplitAddWidget, { index: index, lineNumber: newLine.lineNumber, side: SplitSide.new, diffFile: diffFile, onWidgetClick: onAddWidgetClick, className: "absolute left-[100%] translate-x-[-50%] z-[1]", onOpenAddWidget: (lineNumber, side) => setWidget({ lineNumber: lineNumber, side: side }) })),
                React.createElement("span", { "data-line-num": newLine.lineNumber, style: { opacity: hasChange ? undefined : 0.5 } }, newLine.lineNumber)),
            React.createElement("td", { className: "diff-line-new-content group relative pr-[10px] align-top", "data-side": SplitSide[SplitSide.new], style: { backgroundColor: newLineContentBG } },
                hasDiff && enableAddWidget && (React.createElement(DiffSplitAddWidget, { index: index, lineNumber: newLine.lineNumber, side: SplitSide.new, diffFile: diffFile, onWidgetClick: onAddWidgetClick, className: "absolute right-[100%] translate-x-[50%] z-[1] select-none", onOpenAddWidget: (lineNumber, side) => setWidget({ lineNumber: lineNumber, side: side }) })),
                React.createElement(DiffContent, { enableWrap: true, diffFile: diffFile, rawLine: newLine.value || "", diffLine: newLine.diff, syntaxLine: newSyntaxLine, enableHighlight: enableHighlight })))) : (React.createElement("td", { className: "diff-line-new-placeholder select-none border-l-[1px] border-l-[#ccc]", style: { backgroundColor: `var(${emptyBGName})` }, "data-side": SplitSide[SplitSide.new], colSpan: 2 },
            React.createElement("span", null, "\u2002")))));
};
const DiffSplitLine = ({ index, diffFile, lineNumber, }) => {
    const oldLine = diffFile.getSplitLeftLine(index);
    const newLine = diffFile.getSplitRightLine(index);
    if ((oldLine === null || oldLine === void 0 ? void 0 : oldLine.isHidden) && (newLine === null || newLine === void 0 ? void 0 : newLine.isHidden))
        return null;
    return React.createElement(_DiffSplitLine, { index: index, diffFile: diffFile, lineNumber: lineNumber });
};

const _DiffSplitWidgetLine = ({ index, diffFile, lineNumber, }) => {
    const { useWidget } = useDiffWidgetContext();
    const { widgetLineNumber, widgetSide, setWidget } = useWidget(React.useCallback((s) => ({ widgetLineNumber: s.widgetLineNumber, widgetSide: s.widgetSide, setWidget: s.setWidget }), []));
    const { useDiffContext } = useDiffViewContext();
    const renderWidgetLine = useDiffContext(React.useCallback((s) => s.renderWidgetLine, []));
    const oldLine = diffFile.getSplitLeftLine(index);
    const newLine = diffFile.getSplitRightLine(index);
    const oldLineWidget = oldLine.lineNumber && widgetSide === SplitSide.old && widgetLineNumber === oldLine.lineNumber;
    const newLineWidget = newLine.lineNumber && widgetSide === SplitSide.new && widgetLineNumber === newLine.lineNumber;
    if (!renderWidgetLine)
        return null;
    return (React.createElement("tr", { "data-line": `${lineNumber}-widget`, "data-state": "widget", className: "diff-line diff-line-widget" },
        oldLineWidget ? (React.createElement("td", { className: "diff-line-widget-old-content p-0", colSpan: 2 },
            React.createElement("div", { className: "diff-line-widget-wrapper" }, renderWidgetLine === null || renderWidgetLine === void 0 ? void 0 : renderWidgetLine({
                diffFile,
                side: SplitSide.old,
                lineNumber: oldLine.lineNumber,
                onClose: () => setWidget({}),
            })))) : (React.createElement("td", { className: "diff-line-widget-old-placeholder p-0 select-none", style: { backgroundColor: `var(${emptyBGName})` }, colSpan: 2 },
            React.createElement("span", null, "\u2002"))),
        newLineWidget ? (React.createElement("td", { className: "diff-line-widget-new-content p-0 border-l-[1px] border-l-[#ccc]", colSpan: 2 },
            React.createElement("div", { className: "diff-line-widget-wrapper" }, renderWidgetLine === null || renderWidgetLine === void 0 ? void 0 : renderWidgetLine({
                diffFile,
                side: SplitSide.new,
                lineNumber: newLine.lineNumber,
                onClose: () => setWidget({}),
            })))) : (React.createElement("td", { className: "diff-line-widget-new-placeholder p-0 border-l-[1px] border-l-[#ccc] select-none", style: { backgroundColor: `var(${emptyBGName})` }, colSpan: 2 },
            React.createElement("span", null, "\u2002")))));
};
const DiffSplitWidgetLine = ({ index, diffFile, lineNumber, }) => {
    const { useWidget } = useDiffWidgetContext();
    const { widgetLineNumber, widgetSide } = useWidget(React.useCallback((s) => ({ widgetLineNumber: s.widgetLineNumber, widgetSide: s.widgetSide }), []));
    const oldLine = diffFile.getSplitLeftLine(index);
    const newLine = diffFile.getSplitRightLine(index);
    const oldLineWidget = oldLine.lineNumber && widgetSide === SplitSide.old && widgetLineNumber === oldLine.lineNumber;
    const newLineWidget = newLine.lineNumber && widgetSide === SplitSide.new && widgetLineNumber === newLine.lineNumber;
    const currentIsShow = oldLineWidget || newLineWidget;
    if (!currentIsShow)
        return null;
    return React.createElement(_DiffSplitWidgetLine, { index: index, diffFile: diffFile, lineNumber: lineNumber });
};

const Style = ({ useSelector, id, }) => {
    const splitRef = useSelector((s) => s.splitRef);
    return (React.createElement("style", null, splitRef === SplitSide.old
        ? `#${id} td[data-side="${SplitSide[SplitSide.new]}"] {user-select: none}`
        : splitRef === SplitSide.new
            ? `#${id} td[data-side="${SplitSide[SplitSide.old]}"] {user-select: none}`
            : ""));
};
const DiffSplitViewWrap = memo(({ diffFile }) => {
    const splitLineLength = diffFile.splitLineLength;
    const { useDiffContext } = useDiffViewContext();
    const splitSideInfo = useMemo(() => createStore(() => {
        const splitRef = ref(undefined);
        const setSplit = (side) => {
            flushSync(() => {
                splitRef.value = side;
            });
        };
        return { splitRef, setSplit };
    }), []);
    const setSelectSide = splitSideInfo.getReadonlyState().setSplit;
    const fontSize = useDiffContext(useCallback((s) => s.fontSize, []));
    useSyncExternalStore(diffFile.subscribe, diffFile.getUpdateCount);
    const onMouseDown = useCallback((e) => {
        let ele = e.target;
        // need remove all the selection
        if (ele && ele instanceof HTMLElement && ele.nodeName === "BUTTON") {
            removeAllSelection();
            return;
        }
        while (ele && ele instanceof HTMLElement && ele.nodeName !== "TD") {
            ele = ele.parentElement;
        }
        if (ele instanceof HTMLElement) {
            const side = ele.getAttribute("data-side");
            if (side) {
                setSelectSide(SplitSide[side]);
                removeAllSelection();
            }
        }
    }, [setSelectSide]);
    const width = useTextWidth({
        text: splitLineLength.toString(),
        font: { fontSize: fontSize + "px", fontFamily: "Menlo, Consolas, monospace" },
    });
    const lines = getSplitContentLines(diffFile);
    return (React.createElement("div", { className: "split-diff-view split-diff-view-normal w-full" },
        React.createElement("div", { className: "diff-table-wrapper w-full", style: {
                fontFamily: "Menlo, Consolas, monospace",
                fontSize: "var(--diff-font-size--)",
            } },
            React.createElement(Style, { useSelector: splitSideInfo, id: `diff-root${diffFile.getId()}` }),
            React.createElement("table", { className: "diff-table border-collapse table-fixed w-full" },
                React.createElement("colgroup", null,
                    React.createElement("col", { className: "diff-table-old-num-col", width: Math.round(width) + 25 }),
                    React.createElement("col", { className: "diff-table-old-content-col" }),
                    React.createElement("col", { className: "diff-table-new-num-col", width: Math.round(width) + 25 }),
                    React.createElement("col", { className: "diff-table-new-content-col" })),
                React.createElement("thead", { className: "hidden" },
                    React.createElement("tr", null,
                        React.createElement("th", { scope: "col" }, "old line number"),
                        React.createElement("th", { scope: "col" }, "old line content"),
                        React.createElement("th", { scope: "col" }, "new line number"),
                        React.createElement("th", { scope: "col" }, "new line content"))),
                React.createElement("tbody", { className: "diff-table-body leading-[1.4]", onMouseDownCapture: onMouseDown },
                    lines.map((line) => (React.createElement(Fragment, { key: line.index },
                        React.createElement(DiffSplitHunkLine, { index: line.index, lineNumber: line.lineNumber, diffFile: diffFile }),
                        React.createElement(DiffSplitLine, { index: line.index, lineNumber: line.lineNumber, diffFile: diffFile }),
                        React.createElement(DiffSplitWidgetLine, { index: line.index, lineNumber: line.lineNumber, diffFile: diffFile }),
                        React.createElement(DiffSplitExtendLine, { index: line.index, lineNumber: line.lineNumber, diffFile: diffFile })))),
                    React.createElement(DiffSplitHunkLine, { index: diffFile.splitLineLength, lineNumber: diffFile.splitLineLength, diffFile: diffFile }))))));
});
DiffSplitViewWrap.displayName = "DiffSplitViewWrap";

const DiffSplitView = memo(({ diffFile }) => {
    const { useDiffContext } = useDiffViewContext();
    const enableWrap = useDiffContext(React.useCallback((s) => s.enableWrap, []));
    // performance optimization
    const useWidget = useMemo(() => createStore(() => {
        const widgetSide = ref(undefined);
        const widgetLineNumber = ref(undefined);
        const setWidget = ({ side, lineNumber }) => {
            const { renderWidgetLine } = useDiffContext.getReadonlyState();
            if (typeof renderWidgetLine !== "function")
                return;
            widgetSide.value = side;
            widgetLineNumber.value = lineNumber;
        };
        return { widgetSide, widgetLineNumber, setWidget };
    }), [useDiffContext]);
    const contextValue = useMemo(() => ({ useWidget }), [useWidget]);
    useSyncExternalStore(diffFile.subscribe, diffFile.getUpdateCount);
    useEffect(() => {
        const { setWidget } = useWidget.getReadonlyState();
        setWidget({});
    }, [diffFile, useWidget]);
    return (React.createElement(DiffWidgetContext.Provider, { value: contextValue }, enableWrap ? React.createElement(DiffSplitViewWrap, { diffFile: diffFile }) : React.createElement(DiffSplitViewNormal, { diffFile: diffFile })));
});
DiffSplitView.displayName = "DiffSplitView";

const _DiffUnifiedExtendLine = ({ index, diffFile, lineNumber, }) => {
    const { useDiffContext } = useDiffViewContext();
    const renderExtendLine = useDiffContext(useCallback((s) => s.renderExtendLine, []));
    const unifiedItem = diffFile.getUnifiedLine(index);
    const oldExtend = useDiffContext(useCallback((s) => { var _a, _b; return (_b = (_a = s.extendData) === null || _a === void 0 ? void 0 : _a.oldFile) === null || _b === void 0 ? void 0 : _b[unifiedItem === null || unifiedItem === void 0 ? void 0 : unifiedItem.oldLineNumber]; }, [unifiedItem === null || unifiedItem === void 0 ? void 0 : unifiedItem.oldLineNumber]));
    const newExtend = useDiffContext(useCallback((s) => { var _a, _b; return (_b = (_a = s.extendData) === null || _a === void 0 ? void 0 : _a.newFile) === null || _b === void 0 ? void 0 : _b[unifiedItem === null || unifiedItem === void 0 ? void 0 : unifiedItem.newLineNumber]; }, [unifiedItem === null || unifiedItem === void 0 ? void 0 : unifiedItem.newLineNumber]));
    const width = useDomWidth({
        selector: ".unified-diff-table-wrapper",
        enable: typeof renderExtendLine === "function",
    });
    if (!renderExtendLine)
        return null;
    return (React.createElement("tr", { "data-line": `${lineNumber}-extend`, "data-state": "extend", className: "diff-line diff-line-extend" },
        React.createElement("td", { className: "diff-line-extend-content align-top p-0", colSpan: 2 },
            React.createElement("div", { className: "diff-line-extend-wrapper sticky left-0", style: { width } },
                width > 0 &&
                    oldExtend &&
                    (renderExtendLine === null || renderExtendLine === void 0 ? void 0 : renderExtendLine({
                        diffFile,
                        side: SplitSide.old,
                        lineNumber: unifiedItem.oldLineNumber,
                        data: oldExtend.data,
                        onUpdate: diffFile.notifyAll,
                    })),
                width > 0 &&
                    newExtend &&
                    (renderExtendLine === null || renderExtendLine === void 0 ? void 0 : renderExtendLine({
                        diffFile,
                        side: SplitSide.new,
                        lineNumber: unifiedItem.newLineNumber,
                        data: newExtend.data,
                        onUpdate: diffFile.notifyAll,
                    }))))));
};
const DiffUnifiedExtendLine = ({ index, diffFile, lineNumber, }) => {
    const { useDiffContext } = useDiffViewContext();
    const unifiedItem = diffFile.getUnifiedLine(index);
    const hasExtend = useDiffContext(useCallback((s) => { var _a, _b, _c, _d; return ((_b = (_a = s.extendData) === null || _a === void 0 ? void 0 : _a.oldFile) === null || _b === void 0 ? void 0 : _b[unifiedItem === null || unifiedItem === void 0 ? void 0 : unifiedItem.oldLineNumber]) || ((_d = (_c = s.extendData) === null || _c === void 0 ? void 0 : _c.newFile) === null || _d === void 0 ? void 0 : _d[unifiedItem === null || unifiedItem === void 0 ? void 0 : unifiedItem.newLineNumber]); }, [unifiedItem.oldLineNumber, unifiedItem.newLineNumber]));
    if (!hasExtend || !unifiedItem || unifiedItem.isHidden || !unifiedItem.diff)
        return null;
    return React.createElement(_DiffUnifiedExtendLine, { index: index, diffFile: diffFile, lineNumber: lineNumber });
};

const _DiffUnifiedHunkLine = ({ index, diffFile, lineNumber, }) => {
    var _a;
    const currentHunk = diffFile.getUnifiedHunkLine(index);
    const expandEnabled = diffFile.getExpandEnabled();
    const { useDiffContext } = useDiffViewContext();
    const enableWrap = useDiffContext(useCallback((s) => s.enableWrap, []));
    const couldExpand = expandEnabled && currentHunk && currentHunk.unifiedInfo;
    const isExpandAll = currentHunk &&
        currentHunk.unifiedInfo &&
        currentHunk.unifiedInfo.endHiddenIndex - currentHunk.unifiedInfo.startHiddenIndex < composeLen;
    const isFirstLine = currentHunk && currentHunk.index === 0;
    const isLastLine = currentHunk && currentHunk.isLast;
    return (React.createElement("tr", { "data-line": `${lineNumber}-hunk`, "data-state": "hunk", className: "diff-line diff-line-hunk" },
        React.createElement("td", { className: "diff-line-hunk-action sticky left-0 p-[1px] w-[1%] min-w-[100px] select-none", style: {
                backgroundColor: `var(${hunkLineNumberBGName})`,
                color: `var(${plainLineNumberColorName})`,
            } }, couldExpand ? (isFirstLine ? (React.createElement("button", { className: "w-full diff-widget-tooltip hover:bg-blue-300 flex justify-center items-center py-[6px] cursor-pointer rounded-[2px]", title: "Expand Up", "data-title": "Expand Up", onClick: () => diffFile.onUnifiedHunkExpand("up", index) },
            React.createElement(ExpandUp, { className: "fill-current" }))) : isLastLine ? (React.createElement("button", { className: "w-full diff-widget-tooltip hover:bg-blue-300 flex justify-center items-center py-[6px] cursor-pointer rounded-[2px]", title: "Expand Down", "data-title": "Expand Down", onClick: () => diffFile.onUnifiedHunkExpand("down", index) },
            React.createElement(ExpandDown, { className: "fill-current" }))) : isExpandAll ? (React.createElement("button", { className: "w-full diff-widget-tooltip hover:bg-blue-300 flex justify-center items-center py-[6px] cursor-pointer rounded-[2px]", title: "Expand All", "data-title": "Expand All", onClick: () => diffFile.onUnifiedHunkExpand("all", index) },
            React.createElement(ExpandAll, { className: "fill-current" }))) : (React.createElement(React.Fragment, null,
            React.createElement("button", { className: "w-full diff-widget-tooltip hover:bg-blue-300 flex justify-center items-center py-[2px] cursor-pointer rounded-[2px]", title: "Expand Down", "data-title": "Expand Down", onClick: () => diffFile.onUnifiedHunkExpand("down", index) },
                React.createElement(ExpandDown, { className: "fill-current" })),
            React.createElement("button", { className: "w-full diff-widget-tooltip hover:bg-blue-300 flex justify-center items-center py-[2px] cursor-pointer rounded-[2px]", title: "Expand Up", "data-title": "Expand Up", onClick: () => diffFile.onUnifiedHunkExpand("up", index) },
                React.createElement(ExpandUp, { className: "fill-current" }))))) : (React.createElement("div", { className: "min-h-[28px]" }, "\u2002"))),
        React.createElement("td", { className: "diff-line-hunk-content pr-[10px] align-middle", style: { backgroundColor: `var(${hunkContentBGName})` } },
            React.createElement("div", { className: "pl-[1.5em]", style: {
                    whiteSpace: enableWrap ? "pre-wrap" : "pre",
                    wordBreak: enableWrap ? "break-all" : "initial",
                    color: `var(${hunkContentColorName})`,
                } }, ((_a = currentHunk.unifiedInfo) === null || _a === void 0 ? void 0 : _a.plainText) || currentHunk.text))));
};
const DiffUnifiedHunkLine = ({ index, diffFile, lineNumber, }) => {
    const currentHunk = diffFile.getUnifiedHunkLine(index);
    const currentIsShow = currentHunk &&
        currentHunk.unifiedInfo &&
        currentHunk.unifiedInfo.startHiddenIndex < currentHunk.unifiedInfo.endHiddenIndex;
    const currentIsPureHunk = currentHunk && diffFile._getIsPureDiffRender() && !currentHunk.unifiedInfo;
    if (!currentIsShow && !currentIsPureHunk)
        return null;
    return React.createElement(_DiffUnifiedHunkLine, { index: index, diffFile: diffFile, lineNumber: lineNumber });
};

const DiffUnifiedOldLine = ({ index, diffLine, rawLine, syntaxLine, lineNumber, diffFile, setWidget, enableWrap, enableAddWidget, enableHighlight, onAddWidgetClick, }) => {
    return (React.createElement("tr", { "data-line": index, "data-state": "diff", className: "diff-line group" },
        React.createElement("td", { className: "diff-line-num sticky left-0 pl-[10px] pr-[10px] text-right select-none w-[1%] min-w-[100px] whitespace-nowrap align-top", style: {
                color: `var(${plainLineNumberColorName})`,
                backgroundColor: `var(${delLineNumberBGName})`,
            } },
            enableAddWidget && (React.createElement(DiffUnifiedAddWidget, { index: index - 1, lineNumber: lineNumber, diffFile: diffFile, side: SplitSide.old, onWidgetClick: onAddWidgetClick, onOpenAddWidget: (lineNumber, side) => setWidget({ lineNumber, side }) })),
            React.createElement("div", { className: "flex" },
                React.createElement("span", { "data-line-old-num": lineNumber, className: "inline-block w-[50%]" }, lineNumber),
                React.createElement("span", { className: "w-[10px] shrink-0" }),
                React.createElement("span", { className: "inline-block w-[50%]" }))),
        React.createElement("td", { className: "diff-line-content pr-[10px] align-top", style: { backgroundColor: `var(${delContentBGName})` } },
            React.createElement(DiffContent, { enableWrap: enableWrap, diffFile: diffFile, enableHighlight: enableHighlight, rawLine: rawLine, diffLine: diffLine, syntaxLine: syntaxLine }))));
};
const DiffUnifiedNewLine = ({ index, diffLine, rawLine, syntaxLine, lineNumber, diffFile, setWidget, enableWrap, enableAddWidget, enableHighlight, onAddWidgetClick, }) => {
    return (React.createElement("tr", { "data-line": index, "data-state": "diff", className: "diff-line group" },
        React.createElement("td", { className: "diff-line-num sticky left-0 pl-[10px] pr-[10px] text-right select-none w-[1%] min-w-[100px] whitespace-nowrap align-top", style: {
                color: `var(${plainLineNumberColorName})`,
                backgroundColor: `var(${addLineNumberBGName})`,
            } },
            enableAddWidget && (React.createElement(DiffUnifiedAddWidget, { index: index - 1, lineNumber: lineNumber, diffFile: diffFile, side: SplitSide.new, onWidgetClick: onAddWidgetClick, onOpenAddWidget: (lineNumber, side) => setWidget({ lineNumber, side }) })),
            React.createElement("div", { className: "flex" },
                React.createElement("span", { className: "inline-block w-[50%]" }),
                React.createElement("span", { className: "shrink-0 w-[10px]" }),
                React.createElement("span", { "data-line-new-num": lineNumber, className: "inline-block w-[50%]" }, lineNumber))),
        React.createElement("td", { className: "diff-line-content pr-[10px] align-top", style: { backgroundColor: `var(${addContentBGName})` } },
            React.createElement(DiffContent, { enableWrap: enableWrap, diffFile: diffFile, enableHighlight: enableHighlight, rawLine: rawLine, diffLine: diffLine, syntaxLine: syntaxLine }))));
};
const _DiffUnifiedLine = memo(({ index, diffFile, lineNumber }) => {
    const unifiedLine = diffFile.getUnifiedLine(index);
    const { useDiffContext } = useDiffViewContext();
    const { enableWrap, enableHighlight, enableAddWidget, onAddWidgetClick } = useDiffContext(React.useCallback((s) => ({
        enableWrap: s.enableWrap,
        enableHighlight: s.enableHighlight,
        enableAddWidget: s.enableAddWidget,
        onAddWidgetClick: s.onAddWidgetClick,
    }), []));
    const { useWidget } = useDiffWidgetContext();
    const { setWidget } = useWidget.getReadonlyState();
    const hasDiff = unifiedLine.diff;
    const hasChange = checkDiffLineIncludeChange(unifiedLine.diff);
    const rawLine = unifiedLine.value || "";
    const diffLine = unifiedLine.diff;
    const newLineNumber = unifiedLine.newLineNumber;
    const oldLinenumber = unifiedLine.oldLineNumber;
    const syntaxLine = newLineNumber
        ? diffFile.getNewSyntaxLine(newLineNumber)
        : oldLinenumber
            ? diffFile.getOldSyntaxLine(oldLinenumber)
            : undefined;
    if (hasChange) {
        if (unifiedLine.oldLineNumber) {
            return (React.createElement(DiffUnifiedOldLine, { index: lineNumber, enableWrap: enableWrap, diffFile: diffFile, rawLine: rawLine, diffLine: diffLine, setWidget: setWidget, syntaxLine: syntaxLine, enableHighlight: enableHighlight, enableAddWidget: enableAddWidget, onAddWidgetClick: onAddWidgetClick, lineNumber: unifiedLine.oldLineNumber }));
        }
        else {
            return (React.createElement(DiffUnifiedNewLine, { index: lineNumber, enableWrap: enableWrap, rawLine: rawLine, diffLine: diffLine, diffFile: diffFile, setWidget: setWidget, syntaxLine: syntaxLine, enableHighlight: enableHighlight, enableAddWidget: enableAddWidget, onAddWidgetClick: onAddWidgetClick, lineNumber: unifiedLine.newLineNumber }));
        }
    }
    else {
        return (React.createElement("tr", { "data-line": lineNumber, "data-state": unifiedLine.diff ? "diff" : "plain", className: "diff-line group" },
            React.createElement("td", { className: "diff-line-num sticky left-0 pl-[10px] pr-[10px] text-right select-none w-[1%] min-w-[100px] whitespace-nowrap align-top", style: {
                    color: `var(${plainLineNumberColorName})`,
                    backgroundColor: hasDiff ? `var(${plainLineNumberBGName})` : `var(${expandContentBGName})`,
                } },
                enableAddWidget && hasDiff && (React.createElement(DiffUnifiedAddWidget, { index: index, diffFile: diffFile, lineNumber: unifiedLine.newLineNumber, side: SplitSide.new, onWidgetClick: onAddWidgetClick, onOpenAddWidget: (lineNumber, side) => setWidget({ lineNumber, side }) })),
                React.createElement("div", { className: "flex opacity-[0.5]" },
                    React.createElement("span", { "data-line-old-num": unifiedLine.oldLineNumber, className: "inline-block w-[50%]" }, unifiedLine.oldLineNumber),
                    React.createElement("span", { className: "w-[10px] shrink-0" }),
                    React.createElement("span", { "data-line-new-num": unifiedLine.newLineNumber, className: "inline-block w-[50%]" }, unifiedLine.newLineNumber))),
            React.createElement("td", { className: "diff-line-content pr-[10px] align-top", style: {
                    backgroundColor: hasDiff ? `var(${plainContentBGName})` : `var(${expandContentBGName})`,
                } },
                React.createElement(DiffContent, { enableWrap: enableWrap, diffFile: diffFile, enableHighlight: enableHighlight, rawLine: rawLine, diffLine: diffLine, syntaxLine: syntaxLine }))));
    }
});
_DiffUnifiedLine.displayName = "_DiffUnifiedLine";
const DiffUnifiedLine = ({ index, diffFile, lineNumber, }) => {
    const unifiedLine = diffFile.getUnifiedLine(index);
    if (unifiedLine === null || unifiedLine === void 0 ? void 0 : unifiedLine.isHidden)
        return null;
    return React.createElement(_DiffUnifiedLine, { index: index, diffFile: diffFile, lineNumber: lineNumber });
};

const _DiffUnifiedWidgetLine = ({ index, diffFile, oldWidget, newWidget, lineNumber, setWidget, }) => {
    const unifiedItem = diffFile.getUnifiedLine(index);
    const onClose = () => setWidget({});
    const { useDiffContext } = useDiffViewContext();
    // 需要显示的时候才进行方法订阅，可以大幅度提高性能
    const renderWidgetLine = useDiffContext(useCallback((s) => s.renderWidgetLine, []));
    const width = useDomWidth({
        selector: ".unified-diff-table-wrapper",
        enable: typeof renderWidgetLine === "function",
    });
    if (!renderWidgetLine)
        return null;
    return (React.createElement("tr", { "data-line": `${lineNumber}-widget`, "data-state": "widget", className: "diff-line diff-line-widget" },
        React.createElement("td", { className: "diff-line-widget-content p-0", colSpan: 2 },
            React.createElement("div", { className: "diff-line-widget-wrapper sticky left-0", style: { width } },
                width > 0 &&
                    oldWidget &&
                    (renderWidgetLine === null || renderWidgetLine === void 0 ? void 0 : renderWidgetLine({ diffFile, side: SplitSide.old, lineNumber: unifiedItem.oldLineNumber, onClose })),
                width > 0 &&
                    newWidget &&
                    (renderWidgetLine === null || renderWidgetLine === void 0 ? void 0 : renderWidgetLine({ diffFile, side: SplitSide.new, lineNumber: unifiedItem.newLineNumber, onClose }))))));
};
const DiffUnifiedWidgetLine = ({ index, diffFile, lineNumber, }) => {
    const { useWidget } = useDiffWidgetContext();
    const { widgetSide, widgetLineNumber, setWidget } = useWidget(React.useCallback((s) => ({ widgetLineNumber: s.widgetLineNumber, widgetSide: s.widgetSide, setWidget: s.setWidget }), []));
    const unifiedItem = diffFile.getUnifiedLine(index);
    const oldWidget = unifiedItem.oldLineNumber && widgetSide === SplitSide.old && widgetLineNumber === unifiedItem.oldLineNumber;
    const newWidget = unifiedItem.newLineNumber && widgetSide === SplitSide.new && widgetLineNumber === unifiedItem.newLineNumber;
    const currentIsShow = oldWidget || newWidget;
    if (!currentIsShow)
        return null;
    return (React.createElement(_DiffUnifiedWidgetLine, { index: index, diffFile: diffFile, lineNumber: lineNumber, oldWidget: oldWidget, newWidget: newWidget, setWidget: setWidget }));
};

const onMouseDown = (e) => {
    const ele = e.target;
    // need remove all the selection
    if (ele && ele instanceof HTMLElement && ele.nodeName === "BUTTON") {
        removeAllSelection();
        return;
    }
};
const DiffUnifiedView = memo(({ diffFile }) => {
    const { useDiffContext } = useDiffViewContext();
    // performance optimization
    const useWidget = useMemo(() => createStore(() => {
        const widgetSide = ref(undefined);
        const widgetLineNumber = ref(undefined);
        const setWidget = ({ side, lineNumber }) => {
            const { renderWidgetLine } = useDiffContext.getReadonlyState();
            if (typeof renderWidgetLine !== "function")
                return;
            widgetSide.value = side;
            widgetLineNumber.value = lineNumber;
        };
        return { widgetSide, widgetLineNumber, setWidget };
    }), [useDiffContext]);
    const contextValue = useMemo(() => ({ useWidget }), [useWidget]);
    useSyncExternalStore(diffFile.subscribe, diffFile.getUpdateCount);
    useEffect(() => {
        const { setWidget } = useWidget.getReadonlyState();
        setWidget({});
    }, [diffFile, useWidget]);
    const lines = getUnifiedContentLine(diffFile);
    return (React.createElement(DiffWidgetContext.Provider, { value: contextValue },
        React.createElement("div", { className: "unified-diff-view w-full" },
            React.createElement("div", { className: "unified-diff-table-wrapper overflow-auto w-full scrollbar-hide scrollbar-disable", style: { fontFamily: "Menlo, Consolas, monospace", fontSize: "var(--diff-font-size--)" } },
                React.createElement("table", { className: "unified-diff-table border-collapse w-full" },
                    React.createElement("colgroup", null,
                        React.createElement("col", { className: "unified-diff-table-num-col" }),
                        React.createElement("col", { className: "unified-diff-table-content-col" })),
                    React.createElement("thead", { className: "hidden" },
                        React.createElement("tr", null,
                            React.createElement("th", { scope: "col" }, "line number"),
                            React.createElement("th", { scope: "col" }, "line content"))),
                    React.createElement("tbody", { className: "diff-table-body  leading-[1.4]", onMouseDownCapture: onMouseDown },
                        lines.map((item) => (React.createElement(Fragment, { key: item.index },
                            React.createElement(DiffUnifiedHunkLine, { index: item.index, lineNumber: item.lineNumber, diffFile: diffFile }),
                            React.createElement(DiffUnifiedLine, { index: item.index, lineNumber: item.lineNumber, diffFile: diffFile }),
                            React.createElement(DiffUnifiedWidgetLine, { index: item.index, lineNumber: item.lineNumber, diffFile: diffFile }),
                            React.createElement(DiffUnifiedExtendLine, { index: item.index, lineNumber: item.lineNumber, diffFile: diffFile })))),
                        React.createElement(DiffUnifiedHunkLine, { index: diffFile.unifiedLineLength, lineNumber: diffFile.unifiedLineLength, diffFile: diffFile })))))));
});
DiffUnifiedView.displayName = "DiffUnifiedView";

const diffFontSizeName = "--diff-font-size--";
_cacheMap.name = "@git-diff-view/react";
var SplitSide;
(function (SplitSide) {
    SplitSide[SplitSide["old"] = 1] = "old";
    SplitSide[SplitSide["new"] = 2] = "new";
})(SplitSide || (SplitSide = {}));
const _InternalDiffView = (props) => {
    const { diffFile, className, style, diffViewMode, diffViewWrap, diffViewFontSize, diffViewHighlight, renderWidgetLine, renderExtendLine, extendData, diffViewAddWidget, onAddWidgetClick, } = props;
    const diffFileId = useMemo(() => diffFile.getId(), [diffFile]);
    // performance optimization
    const useDiffContext = useMemo(() => createStore(() => {
        var _a, _b;
        const id = ref(diffFileId);
        const setId = (_id) => (id.value = _id);
        const mode = ref(props.diffViewMode);
        const setMode = (_mode) => (mode.value = _mode);
        const enableWrap = ref(props.diffViewWrap);
        const setEnableWrap = (_enableWrap) => (enableWrap.value = _enableWrap);
        const enableAddWidget = ref(props.diffViewAddWidget);
        const setEnableAddWidget = (_enableAddWidget) => (enableAddWidget.value = _enableAddWidget);
        const enableHighlight = ref(props.diffViewHighlight);
        const setEnableHighlight = (_enableHighlight) => (enableHighlight.value = _enableHighlight);
        const fontSize = ref(props.diffViewFontSize);
        const setFontSize = (_fontSize) => (fontSize.value = _fontSize);
        const extendData = ref({
            oldFile: Object.assign({}, (_a = props.extendData) === null || _a === void 0 ? void 0 : _a.oldFile),
            newFile: Object.assign({}, (_b = props.extendData) === null || _b === void 0 ? void 0 : _b.newFile),
        });
        const setExtendData = (_extendData) => {
            const existOldKeys = Object.keys(extendData.value.oldFile || {});
            const inComingOldKeys = Object.keys(_extendData.oldFile || {});
            for (const key of existOldKeys) {
                if (!inComingOldKeys.includes(key)) {
                    delete extendData.value.oldFile[key];
                }
            }
            for (const key of inComingOldKeys) {
                extendData.value.oldFile[key] = _extendData.oldFile[key];
            }
            const existNewKeys = Object.keys(extendData.value.newFile || {});
            const inComingNewKeys = Object.keys(_extendData.newFile || {});
            for (const key of existNewKeys) {
                if (!inComingNewKeys.includes(key)) {
                    delete extendData.value.newFile[key];
                }
            }
            for (const key of inComingNewKeys) {
                extendData.value.newFile[key] = _extendData.newFile[key];
            }
        };
        const renderWidgetLine = ref(props.renderWidgetLine);
        const setRenderWidgetLine = (_renderWidgetLine) => (renderWidgetLine.value = _renderWidgetLine);
        const renderExtendLine = ref(props.renderExtendLine);
        const setRenderExtendLine = (_renderExtendLine) => (renderExtendLine.value = _renderExtendLine);
        const onAddWidgetClick = ref(props.onAddWidgetClick);
        const setOnAddWidgetClick = (_onAddWidgetClick) => (onAddWidgetClick.value = _onAddWidgetClick);
        return {
            id,
            setId,
            mode,
            setMode,
            enableWrap,
            setEnableWrap,
            enableAddWidget,
            setEnableAddWidget,
            enableHighlight,
            setEnableHighlight,
            fontSize,
            setFontSize,
            extendData,
            setExtendData,
            renderWidgetLine,
            setRenderWidgetLine,
            renderExtendLine,
            setRenderExtendLine,
            onAddWidgetClick,
            setOnAddWidgetClick,
        };
    }), 
    // eslint-disable-next-line react-hooks/exhaustive-deps
    []);
    useEffect(() => {
        const { id, setId, mode, setMode, enableAddWidget, setEnableAddWidget, enableHighlight, setEnableHighlight, enableWrap, setEnableWrap, setExtendData, fontSize, setFontSize, onAddWidgetClick: _onAddWidgetClick, setOnAddWidgetClick, renderExtendLine: _renderExtendLine, setRenderExtendLine, renderWidgetLine: _renderWidgetLine, setRenderWidgetLine, } = useDiffContext.getReadonlyState();
        diffFileId && diffFileId !== id && setId(diffFileId);
        diffViewMode && diffViewMode !== mode && setMode(diffViewMode);
        diffViewAddWidget !== enableAddWidget && setEnableAddWidget(diffViewAddWidget);
        diffViewHighlight !== enableHighlight && setEnableHighlight(diffViewHighlight);
        diffViewWrap !== enableWrap && setEnableWrap(diffViewWrap);
        extendData && setExtendData(extendData);
        diffViewFontSize && diffViewFontSize !== fontSize && setFontSize(diffViewFontSize);
        onAddWidgetClick !== _onAddWidgetClick && setOnAddWidgetClick(onAddWidgetClick);
        renderExtendLine !== _renderExtendLine && setRenderExtendLine(renderExtendLine);
        renderWidgetLine !== _renderWidgetLine && setRenderWidgetLine(renderWidgetLine);
    }, [
        useDiffContext,
        diffViewFontSize,
        diffViewHighlight,
        diffViewMode,
        diffViewWrap,
        diffViewAddWidget,
        diffFileId,
        renderWidgetLine,
        renderExtendLine,
        extendData,
        onAddWidgetClick,
    ]);
    const value = useMemo(() => ({ useDiffContext }), [useDiffContext]);
    return (React.createElement(DiffViewContext.Provider, { value: value },
        React.createElement("div", { className: "diff-tailwindcss-wrapper", "data-component": "git-diff-view", "data-version": `${"0.0.7"}` },
            React.createElement("div", { className: "diff-style-root", style: {
                    // @ts-ignore
                    [diffFontSizeName]: diffViewFontSize + "px",
                } },
                React.createElement("div", { id: `diff-root${diffFileId}`, className: "diff-view-wrapper" + (className ? ` ${className}` : ""), style: style }, diffViewMode === DiffModeEnum.Split ? (React.createElement(DiffSplitView, { diffFile: diffFile })) : (React.createElement(DiffUnifiedView, { diffFile: diffFile })))))));
};
const InternalDiffView = memo(_InternalDiffView);
const DiffViewWithRef = (props, ref) => {
    const { registerHighlighter, autoDetectLang, data, diffFile: _diffFile } = props, restProps = __rest(props, ["registerHighlighter", "autoDetectLang", "data", "diffFile"]);
    const diffFile = useMemo(() => {
        var _a, _b, _c, _d, _e, _f;
        if (_diffFile) {
            const diffFile = DiffFile.createInstance({});
            diffFile._mergeFullBundle(_diffFile._getFullBundle());
            return diffFile;
        }
        else if (data) {
            return new DiffFile(((_a = data === null || data === void 0 ? void 0 : data.oldFile) === null || _a === void 0 ? void 0 : _a.fileName) || "", ((_b = data === null || data === void 0 ? void 0 : data.oldFile) === null || _b === void 0 ? void 0 : _b.content) || "", ((_c = data === null || data === void 0 ? void 0 : data.newFile) === null || _c === void 0 ? void 0 : _c.fileName) || "", ((_d = data === null || data === void 0 ? void 0 : data.newFile) === null || _d === void 0 ? void 0 : _d.content) || "", (data === null || data === void 0 ? void 0 : data.hunks) || [], ((_e = data === null || data === void 0 ? void 0 : data.oldFile) === null || _e === void 0 ? void 0 : _e.fileLang) || "", ((_f = data === null || data === void 0 ? void 0 : data.newFile) === null || _f === void 0 ? void 0 : _f.fileLang) || "");
        }
        return null;
    }, [data, _diffFile]);
    useEffect(() => {
        if (!diffFile)
            return;
        diffFile.initRaw();
        diffFile.buildSplitDiffLines();
        diffFile.buildUnifiedDiffLines();
    }, [diffFile]);
    useEffect(() => {
        if (!diffFile)
            return;
        if (props.diffViewHighlight) {
            diffFile.initSyntax({ autoDetectLang, registerHighlighter });
            diffFile.notifyAll();
        }
    }, [diffFile, props.diffViewHighlight, autoDetectLang, registerHighlighter]);
    useEffect(() => {
        if (_diffFile && diffFile) {
            _diffFile._addClonedInstance(diffFile);
            return () => {
                _diffFile._delClonedInstance(diffFile);
            };
        }
    }, [diffFile, _diffFile]);
    useUnmount(() => diffFile._destroy(), [diffFile]);
    useImperativeHandle(ref, () => ({ getDiffFileInstance: () => diffFile }), [diffFile]);
    if (!diffFile)
        return null;
    return (React.createElement(InternalDiffView, Object.assign({ key: diffFile.getId() }, restProps, { diffFile: diffFile, diffViewFontSize: restProps.diffViewFontSize || 14 })));
};
const DiffView = forwardRef(DiffViewWithRef);
DiffView.displayName = "DiffView";
const version = "0.0.7";

export { DefaultDiffExpansionStep, DiffFile, DiffFileLineType, DiffLine, DiffLineType, DiffModeEnum, DiffView, DiffViewContext, File, SplitSide, _cacheMap, assertNever, checkDiffLineIncludeChange, composeLen, getDiffRange, getFile, getHunkHeaderExpansionType, getLang, getLargestLineNumber, getSplitContentLines, getSplitLines, getUnifiedContentLine, getUnifiedLines, hasRelativeChange, highlighter, numIterator, relativeChanges, useDiffViewContext, version, versions };
//# sourceMappingURL=index.mjs.map
